from leapp.actors import Actor
from leapp.models import RpmTransactionTasks, SCTPConfig
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


def test_sctp_wanted(current_actor_context):
    current_actor_context.feed(SCTPConfig(wanted=True))
    current_actor_context.run()
    assert current_actor_context.consume(RpmTransactionTasks)
    assert current_actor_context.consume(RpmTransactionTasks)[0].to_install == ['kernel-modules-extra']


def test_sctp_empty_config(current_actor_context):
    current_actor_context.feed(SCTPConfig())
    current_actor_context.run()
    assert not current_actor_context.consume(RpmTransactionTasks)


def test_sctp_no_config(current_actor_context):
    current_actor_context.run()
    assert not current_actor_context.consume(RpmTransactionTasks)


def test_sctp_unwanted(current_actor_context):
    current_actor_context.feed(SCTPConfig(wanted=False))
    current_actor_context.run()
    assert not current_actor_context.consume(RpmTransactionTasks)



import os
import shutil
from distutils.version import LooseVersion

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common import dnfplugin, mounting
from leapp.libraries.common.config.version import get_target_major_version
from leapp.libraries.stdlib import api, CalledProcessError
from leapp.models import RequiredUpgradeInitramPackages  # deprecated
from leapp.models import UpgradeDracutModule  # deprecated
from leapp.models import (
    BootContent,
    TargetOSInstallationImage,
    TargetUserSpaceInfo,
    TargetUserSpaceUpgradeTasks,
    UpgradeInitramfsTasks,
    UsedTargetRepositories
)
from leapp.utils.deprecation import suppress_deprecation

INITRAM_GEN_SCRIPT_NAME = 'generate-initram.sh'
DRACUT_DIR = '/dracut'
DEDICATED_LEAPP_PART_URL = 'https://access.redhat.com/solutions/7011704'


def _get_target_kernel_version(context):
    """
    Get the version of the most recent kernel version within the container.
    """

    kernel_version = None
    try:
        results = context.call(['rpm', '-qa', 'kernel-core'], split=True)

        versions = [ver.replace('kernel-core-', '') for ver in results['stdout']]
        api.current_logger().debug(
            'Versions detected {versions}.'
            .format(versions=versions))
        sorted_versions = sorted(versions, key=LooseVersion, reverse=True)
        kernel_version = next(iter(sorted_versions), None)
    except CalledProcessError:
        raise StopActorExecutionError(
            'Cannot get version of the installed kernel.',
            details={'Problem': 'Could not query the currently installed kernel through rmp.'})

    if not kernel_version:
        raise StopActorExecutionError(
            'Cannot get version of the installed kernel.',
            details={'Problem': 'A rpm query for the available kernels did not produce any results.'})

    return kernel_version


def _get_target_kernel_modules_dir(context):
    """
    Return the path where the custom kernel modules should be copied.
    """

    kernel_version = _get_target_kernel_version(context)
    modules_dir = os.path.join('/', 'lib', 'modules', kernel_version, 'extra', 'leapp')

    return modules_dir


def _reinstall_leapp_repository_hint():
    """
    Convenience function for creating a detail for StopActorExecutionError with a hint to reinstall the
    leapp-repository package
    """
    return {
        'hint': 'Try to reinstall the `leapp-repository` package.'
    }


def _copy_modules(context, modules, dst_dir, kind):
    """
    Copy modules of given kind to the specified destination directory.

    Attempts to remove an cleanup by removing the existing destination
    directory. If the directory does not exist, it is created anew. Then, for
    each module message, it checks if the module has a module path specified. If
    the module already exists in the destination directory, a debug message is
    logged, and the operation is skipped. Otherwise, the module is copied to the
    destination directory.

    """

    try:
        context.remove_tree(dst_dir)
    except EnvironmentError:
        pass

    context.makedirs(dst_dir)

    for module in modules:
        if not module.module_path:
            continue

        dst_path = os.path.join(dst_dir, os.path.basename(module.module_path))
        if os.path.exists(context.full_path(dst_path)):
            api.current_logger().debug(
                'The {name} {kind} module has been already installed. Skipping.'
                .format(name=module.name, kind=kind))
            continue

        copy_fn = context.copytree_to
        if os.path.isfile(module.module_path):
            copy_fn = context.copy_to

        try:
            api.current_logger().debug(
                'Copying {kind} module "{name}" to "{path}".'
                .format(kind=kind, name=module.name, path=dst_path))

            copy_fn(module.module_path, dst_path)
        except shutil.Error as e:
            api.current_logger().error(
                    'Failed to copy {kind} module "{name}" from "{source}" to "{target}"'.format(
                        kind=kind, name=module.name, source=module.module_path, target=context.full_path(dst_dir)),
                    exc_info=True)
            raise StopActorExecutionError(
                message='Failed to install {kind} modules required in the initram. Error: {error}'.format(
                    kind=kind, error=str(e))
            )


def copy_dracut_modules(context, modules):
    """
    Copy dracut modules into the target userspace.

    If a module cannot be copied, an error message is logged, and a
    StopActorExecutionError exception is raised.
    """

    _copy_modules(context, modules, DRACUT_DIR, 'dracut')


def copy_kernel_modules(context, modules):
    """
    Copy kernel modules into the target userspace.

    If a module cannot be copied, an error message is logged, and a
    StopActorExecutionError exception is raised.
    """

    dst_dir = _get_target_kernel_modules_dir(context)
    _copy_modules(context, modules, dst_dir, 'kernel')


@suppress_deprecation(UpgradeDracutModule)
def _get_dracut_modules():
    return list(api.consume(UpgradeDracutModule))


def _install_initram_deps(packages):
    used_repos = api.consume(UsedTargetRepositories)
    target_userspace_info = next(api.consume(TargetUserSpaceInfo), None)

    dnfplugin.install_initramdisk_requirements(
        packages=packages,
        target_userspace_info=target_userspace_info,
        used_repos=used_repos)


# duplicate of _copy_files from userspacegen.py
def _copy_files(context, files):
    """
    Copy the files/dirs from the host to the `context` userspace

    :param context: An instance of a mounting.IsolatedActions class
    :type context: mounting.IsolatedActions class
    :param files: list of files that should be copied from the host to the context
    :type files: list of CopyFile
    """
    for file_task in files:
        if not file_task.dst:
            file_task.dst = file_task.src
        if os.path.isdir(file_task.src):
            context.remove_tree(file_task.dst)
            context.copytree_to(file_task.src, file_task.dst)
        else:
            context.copy_to(file_task.src, file_task.dst)


# TODO(pstodulk): think about possibility to split this part to different actor
# # reasoning: the environment could be prepared automatically and actor
# # developers will be able to do additional modifications before the initrd
# # will be really generated. E.g. multipath: config files will be copied
# # and another actor can securely updated configuration before the initrd
# # will be generated. same could be from user's POV - they are not allowed to
# # modify our actors, but they could need to do additional actions inside the
# # env as well.
@suppress_deprecation(RequiredUpgradeInitramPackages)
def prepare_userspace_for_initram(context):
    """
    Prepare the target userspace container to be able to generate init ramdisk

    This includes installation of rpms that are not installed yet. Copying
    files from the host to container, ... So when we start the process of
    the upgrade init ramdisk creation, the environment will be prepared with
    all required data and utilities.

    Note: preparation of dracut modules are handled outside of this function
    """
    packages = set()
    files = []
    _cftuples = set()

    def _update_files(copy_files):
        # add just uniq CopyFile objects to omit duplicate copying of files
        for cfile in copy_files:
            cftuple = (cfile.src, cfile.dst)
            if cftuple not in _cftuples:
                _cftuples.add(cftuple)
                files.append(cfile)

    generator_script = api.get_actor_file_path(INITRAM_GEN_SCRIPT_NAME)
    if not generator_script:
        raise StopActorExecutionError(
            message='Mandatory script to generate initram not available.',
            details=_reinstall_leapp_repository_hint()
        )
    context.copy_to(generator_script, os.path.join('/', INITRAM_GEN_SCRIPT_NAME))
    for msg in api.consume(TargetUserSpaceUpgradeTasks):
        packages.update(msg.install_rpms)
        _update_files(msg.copy_files)
    for message in api.consume(RequiredUpgradeInitramPackages):
        packages.update(message.packages)
    # install all required rpms first, so files installed/copied later
    # will not be overwritten during the dnf transaction
    _install_initram_deps(packages)
    _copy_files(context, files)


def _get_fspace(path, convert_to_mibs=False, coefficient=1):
    """
    Return the free disk space on given path.

    The default is in bytes, but if convert_to_mibs is True, return MiBs instead.

    Raises OSError if nothing exists on the given `path`.

    :param path: Path to an existing file or directory
    :type path: str
    :param convert_to_mibs: If True, convert the value to MiBs
    :type convert_to_mibs: bool
    :param coefficient: Coefficient to multiply the free space (e.g. 0.9 to have it 10% lower). Max: 1
    :type coefficient: float
    :rtype: int
    """
    # TODO(pstodulk): discuss the function params
    # NOTE(pstodulk): This func is copied from the overlaygen.py lib
    # probably it would make sense to make it public in the utils.py lib,
    # but for now, let's keep it private
    stat = os.statvfs(path)

    coefficient = min(coefficient, 1)
    fspace_bytes = int(stat.f_frsize * stat.f_bavail * coefficient)
    if convert_to_mibs:
        return int(fspace_bytes / 1024 / 1024)  # noqa: W1619; pylint: disable=old-division
    return fspace_bytes


def _check_free_space(context):
    """
    Raise StopActorExecutionError if there is less than 500MB of free space available.

    If there is not enough free space in the context, the initramfs will not be
    generated successfully and it's hard to discover what was the issue. Also
    the missing space is able to kill the leapp itself - trying to write to the
    leapp.db when the FS hosting /var/lib/leapp is full, kills the framework
    and the actor execution too - so there is no gentle way to handle such
    exceptions when it happens. From this point, let's rather check the available
    space in advance and stop the execution when it happens.

    It is not expected to hit this issue, but I was successful and I know
    it's still possible even with all other changes (just it's much harder
    now to hit it). So adding this seatbelt, that is not 100% bulletproof,
    but I call it good enough.

    Currently protecting last 500MB. In case of problems, we can increase
    the value.
    """
    message = 'There is not enough space on the file system hosting /var/lib/leapp.'
    hint = (
        'Increase the free space on the filesystem hosting'
        ' /var/lib/leapp by 500MB at minimum (suggested 1500MB).\n\n'
        'It is also a good practice to create dedicated partition'
        ' for /var/lib/leapp when more space is needed, which can be'
        ' dropped after the system upgrade is fully completed.'
        ' For more info, see: {}'
        .format(DEDICATED_LEAPP_PART_URL)
    )
    detail = (
        'Remaining free space is lower than 500MB which is not enough to'
        ' be able to generate the upgrade initramfs. '
    )

    if _get_fspace(context.base_dir, convert_to_mibs=True) < 500:
        raise StopActorExecutionError(
            message=message,
            details={'hint': hint, 'detail': detail}
        )


def generate_initram_disk(context):
    """
    Function to actually execute the init ramdisk creation.

    Includes handling of specified dracut and kernel modules from the host when
    needed. The check for the 'conflicting' modules is in a separate actor.
    """
    _check_free_space(context)
    env = {}
    if get_target_major_version() == '9':
        env = {'SYSTEMD_SECCOMP': '0'}

    # TODO(pstodulk): Add possibility to add particular drivers
    # Issue #645
    modules = {
        'dracut': _get_dracut_modules(),  # deprecated
        'kernel': [],
    }
    files = set()
    for task in api.consume(UpgradeInitramfsTasks):
        modules['dracut'].extend(task.include_dracut_modules)
        modules['kernel'].extend(task.include_kernel_modules)
        files.update(task.include_files)

    copy_dracut_modules(context, modules['dracut'])
    copy_kernel_modules(context, modules['kernel'])

    # FIXME: issue #376
    context.call([
        '/bin/sh', '-c',
        'LEAPP_KERNEL_VERSION={kernel_version} '
        'LEAPP_ADD_DRACUT_MODULES="{dracut_modules}" LEAPP_KERNEL_ARCH={arch} '
        'LEAPP_ADD_KERNEL_MODULES="{kernel_modules}" '
        'LEAPP_DRACUT_INSTALL_FILES="{files}" {cmd}'.format(
            kernel_version=_get_target_kernel_version(context),
            dracut_modules=','.join([mod.name for mod in modules['dracut']]),
            kernel_modules=','.join([mod.name for mod in modules['kernel']]),
            arch=api.current_actor().configuration.architecture,
            files=' '.join(files),
            cmd=os.path.join('/', INITRAM_GEN_SCRIPT_NAME))
    ], env=env)

    copy_boot_files(context)


def create_upgrade_hmac_from_target_hmac(original_hmac_path, upgrade_hmac_path, upgrade_kernel):
    # Rename the kernel name stored in the HMAC file as the upgrade kernel is named differently and the HMAC file
    # refers to the real target kernel
    with open(original_hmac_path) as original_hmac_file:
        hmac_file_lines = [line for line in original_hmac_file.read().split('\n') if line]
        if len(hmac_file_lines) > 1:
            details = ('Expected the target kernel HMAC file to containing only one HMAC line, '
                       'found {0}'.format(len(hmac_file_lines)))
            raise StopActorExecutionError('Failed to prepare HMAC file for upgrade kernel.',
                                          details={'details': details})

        # Keep only non-empty strings after splitting on space
        hmac, dummy_target_kernel_name = [fragment for fragment in hmac_file_lines[0].split(' ') if fragment]

    with open(upgrade_hmac_path, 'w') as upgrade_kernel_hmac_file:
        upgrade_kernel_hmac_file.write('{hmac}  {kernel}\n'.format(hmac=hmac, kernel=upgrade_kernel))


def copy_boot_files(context):
    """
    Function to copy the generated initram and corresponding kernel to /boot - Additionally produces a BootContent
    message with their location.
    """
    curr_arch = api.current_actor().configuration.architecture
    kernel = 'vmlinuz-upgrade.{}'.format(curr_arch)
    initram = 'initramfs-upgrade.{}.img'.format(curr_arch)

    kernel_hmac = '.{0}.hmac'.format(kernel)
    kernel_hmac_path = os.path.join('/boot', kernel_hmac)

    content = BootContent(
        kernel_path=os.path.join('/boot', kernel),
        initram_path=os.path.join('/boot', initram),
        kernel_hmac_path=kernel_hmac_path
    )

    context.copy_from(os.path.join('/artifacts', kernel), content.kernel_path)
    context.copy_from(os.path.join('/artifacts', initram), content.initram_path)

    kernel_hmac_path = context.full_path(os.path.join('/artifacts', kernel_hmac))
    create_upgrade_hmac_from_target_hmac(kernel_hmac_path, content.kernel_hmac_path, kernel)

    api.produce(content)


def process():
    userspace_info = next(api.consume(TargetUserSpaceInfo), None)
    target_iso = next(api.consume(TargetOSInstallationImage), None)
    with mounting.NspawnActions(base_dir=userspace_info.path) as context:
        with mounting.mount_upgrade_iso_to_root_dir(userspace_info.path, target_iso):
            prepare_userspace_for_initram(context)
            generate_initram_disk(context)



import pytest

from leapp import reporting
from leapp.libraries.actor import checkhybridimage
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked, produce_mocked
from leapp.libraries.stdlib import api
from leapp.models import FirmwareFacts, InstalledRPM, RPM
from leapp.reporting import Report

RH_PACKAGER = 'Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>'
WA_AGENT_RPM = RPM(
    name='WALinuxAgent', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
    pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 199e2f91fd431d51'
)
NO_AGENT_RPM = RPM(
    name='NoAgent', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
    pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 199e2f91fd431d51'
)

INSTALLED_AGENT = InstalledRPM(items=[WA_AGENT_RPM])
NOT_INSTALLED_AGENT = InstalledRPM(items=[NO_AGENT_RPM])

BIOS_FIRMWARE = FirmwareFacts(firmware='bios')
EFI_FIRMWARE = FirmwareFacts(firmware='efi')

BIOS_PATH = '/boot/grub2/grubenv'
EFI_PATH = '/boot/efi/EFI/redhat/grubenv'


def test_hybrid_image(monkeypatch, tmpdir):
    grubenv_efi = tmpdir.join('grubenv_efi')
    grubenv_efi.write('grubenv')

    grubenv_boot = tmpdir.join('grubenv_boot')
    grubenv_boot.mksymlinkto('grubenv_efi')

    monkeypatch.setattr(checkhybridimage, 'BIOS_PATH', grubenv_boot.strpath)
    monkeypatch.setattr(checkhybridimage, 'EFI_PATH', grubenv_efi.strpath)
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr(
        api, 'current_actor', CurrentActorMocked(arch='x86_64', msgs=[BIOS_FIRMWARE, INSTALLED_AGENT])
    )
    monkeypatch.setattr(api, "produce", produce_mocked())

    checkhybridimage.check_hybrid_image()
    assert reporting.create_report.called == 1
    assert 'hybrid' in reporting.create_report.report_fields['title']
    assert api.produce.called == 1


@pytest.mark.parametrize('is_symlink, realpath_match, is_bios, agent_installed', [
    (False, True, True, True),
    (True, False, True, True),
    (True, True, False, True),
    (True, True, True, False),
])
def test_no_hybrid_image(monkeypatch, is_symlink, realpath_match, is_bios, agent_installed, tmpdir):
    grubenv_efi = tmpdir.join('grubenv_efi')
    grubenv_efi.write('grubenv')
    grubenv_efi_false = tmpdir.join('grubenv_efi_false')
    grubenv_efi.write('nope')
    grubenv_boot = tmpdir.join('grubenv_boot')

    grubenv_target = grubenv_efi if realpath_match else grubenv_efi_false

    if is_symlink:
        grubenv_boot.mksymlinkto(grubenv_target)

    firmw = BIOS_FIRMWARE if is_bios else EFI_FIRMWARE
    inst_rpms = INSTALLED_AGENT if agent_installed else NOT_INSTALLED_AGENT

    monkeypatch.setattr(checkhybridimage, 'BIOS_PATH', grubenv_boot.strpath)
    monkeypatch.setattr(checkhybridimage, 'EFI_PATH', grubenv_efi.strpath)
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr(
        api, 'current_actor', CurrentActorMocked(arch='x86_64', msgs=[firmw, inst_rpms])
    )
    monkeypatch.setattr(api, "produce", produce_mocked())

    checkhybridimage.check_hybrid_image()
    assert not reporting.create_report.called
    assert not api.produce.called



import re

from leapp import reporting
from leapp.libraries.stdlib import api, run

COMMON_REPORT_TAGS = [reporting.Groups.SERVICES]


sysconfig_path = '/etc/sysconfig/memcached'

related = [
    reporting.RelatedResource('package', 'memcached'),
    reporting.RelatedResource('file', sysconfig_path)
]


def is_sysconfig_default():
    """Check if the memcached sysconfig file was not modified since installation."""
    try:
        result = run(['rpm', '-V', '--nomtime', 'memcached'], checked=False)
        return sysconfig_path not in result['stdout']
    except OSError as e:
        api.current_logger().warning("rpm verification failed: %s", str(e))
        return True


def is_udp_disabled():
    """Check if UDP port is disabled in the sysconfig file."""
    with open(sysconfig_path) as f:
        for line in f:
            if re.match(r'^\s*OPTIONS=.*-U\s*0[^0-9]', line):
                return True
    return False


def check_memcached(memcached_installed):
    """Report potential issues in memcached configuration."""
    if not memcached_installed:
        api.current_logger().info('memcached package is not installed')
        return

    default_memcached_conf = is_sysconfig_default()
    disabled_udp_port = is_udp_disabled()

    if default_memcached_conf:
        reporting.create_report([
            reporting.Title('memcached service is using default configuration'),
            reporting.Summary('memcached in RHEL8 listens on loopback only and has UDP port disabled by default'),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups(COMMON_REPORT_TAGS),
        ] + related)

    elif not disabled_udp_port:
        reporting.create_report([
            reporting.Title('memcached has enabled UDP port'),
            reporting.Summary(
                'memcached in RHEL7 has UDP port enabled by default, but it is disabled by default in RHEL8'
            ),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups(COMMON_REPORT_TAGS),
        ] + related)

    else:
        reporting.create_report([
            reporting.Title('memcached has already disabled UDP port'),
            reporting.Summary('memcached in RHEL8 has UDP port disabled by default'),
            reporting.Severity(reporting.Severity.LOW),
            reporting.Groups(COMMON_REPORT_TAGS),
        ] + related)



from leapp import reporting
from leapp.libraries.stdlib import api
from leapp.models import InstalledDesktopsFacts, InstalledKdeAppsFacts


def check_kde_gnome():
    desktop_facts = next(api.consume(InstalledDesktopsFacts))
    kde_desktop_installed = desktop_facts.kde_installed
    gnome_desktop_installed = desktop_facts.gnome_installed

    # No desktop installed, we don't even care about apps as they are most likely not used or even installed
    if not kde_desktop_installed and not gnome_desktop_installed:
        api.current_logger().info("No desktop installed. Continuing with the upgrade.")
        return

    if kde_desktop_installed:
        api.current_logger().info("KDE desktop is installed. Checking what we can do about it.")
        if not gnome_desktop_installed:
            api.current_logger().error("Cannot perform the upgrade because there is"
                                       " no other desktop than KDE installed.")
            # We cannot continue with the upgrade process
            reporting.create_report([
                reporting.Title("The installed KDE environment is unavailable on RHEL 8."),
                reporting.Summary(
                    "Because the KDE desktop environment is not available on RHEL 8, all the KDE-related packages"
                    " would be removed during the upgrade. There would be no desktop environment installed after the"
                    " upgrade."),
                reporting.Severity(reporting.Severity.HIGH),
                reporting.Groups([
                    reporting.Groups.DESKTOP
                ]),
                reporting.Groups([
                    reporting.Groups.INHIBITOR
                ]),
                reporting.Remediation(
                    hint=("Remove KDE (at least the `kde-workspace` package) or install the GNOME desktop environment"
                          " to be able to upgrade."),
                    commands=[['yum', '-y', 'groupinstall', '"Server with GUI"']])
                ])
            return

        # Assume both GNOME and KDE are installed in this state
        api.current_logger().info("Upgrade can be performed, but KDE desktop will"
                                  " be removed in favor of GNOME")
        reporting.create_report([
            reporting.Title("Upgrade can be performed, but KDE will be uninstalled."),
            reporting.Summary("The KDE desktop environment is unavailable on RHEL 8. KDE will be uninstalled "
                              "in favor of GNOME during the upgrade."),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups([
                reporting.Groups.DESKTOP
            ])])
        api.current_logger().info("----------------------------------")

    # At this state we just need to detect whether any KDE/Qt app is installed to inform user
    # that the application will be removed during the upgrade process. No matter if KDE is installed
    # or not.

    KDEAppsFacts = next(api.consume(InstalledKdeAppsFacts))
    if KDEAppsFacts.installed_apps:
        # upgrade can be performed, but user will loose KDE apps
        api.current_logger().info("Installed KDE/Qt apps detected.")
        reporting.create_report([
            reporting.Title("Upgrade can be performed, but KDE/Qt apps will be uninstalled."),
            reporting.Summary("The KDE desktop environment is unavailable on RHEL 8. "
                              "All the KDE/Qt apps will be removed during the upgrade, including but not limited "
                              "to:\n- {0}".format("\n- ".join(KDEAppsFacts.installed_apps))),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups([
                reporting.Groups.DESKTOP
            ])])
    else:
        api.current_logger().info("No KDE app in use detected.")
        # upgrade can be performed



import pytest

from leapp.libraries.actor.cupscheck import make_reports


class MockLogger(object):
    def __init__(self):
        self.debug_msg = ''

    def debug_log(self, msg):
        self.debug_msg += msg


class MockInputFacts(object):
    def __init__(self, facts):
        self.facts = facts

    def get_facts(self, model):
        ret = None
        if model == 'CupsChangedFeatures':
            ret = self.facts

        return ret


class MockReport(object):
    def __init__(self):
        self.report = []

    # unused, report testing will be done separately
    def create_report(self, data_list):
        if data_list:
            self.report.append(data_list)


def test_make_reports():

    logger = MockLogger()
    facts = MockInputFacts(None)
    reporting = MockReport()

    make_reports(facts.get_facts, reporting.create_report, logger.debug_log)

    assert logger.debug_msg == 'No facts gathered about CUPS - skipping reports.'



from leapp import reporting
from leapp.actors import Actor
from leapp.libraries.actor import iscmodel
from leapp.libraries.stdlib import api
from leapp.models import BindFacts, DistributionSignedRPM
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckBind(Actor):
    """Actor parsing BIND configuration and checking for known issues in it."""

    name = 'check_bind'
    consumes = (DistributionSignedRPM,)
    produces = (BindFacts, reporting.Report)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    pkg_names = {'bind', 'bind-sdb', 'bind-pkcs11'}

    def has_package(self, t_rpms):
        """Replacement for broken leapp.libraries.common.rpms.has_package."""
        for fact in self.consume(t_rpms):
            for rpm in fact.items:
                if rpm.name in self.pkg_names:
                    return True
        return False

    def process(self):
        if not self.has_package(DistributionSignedRPM):
            self.log.debug('bind is not installed')
            return

        facts = iscmodel.get_facts('/etc/named.conf')
        report = iscmodel.make_report(facts)

        if report:
            api.produce(facts)
            self.log.info('BIND configuration issues were found.')
            reporting.create_report(report)
        else:
            self.log.debug('BIND configuration seems compatible.')



from leapp import reporting
from leapp.actors import Actor
from leapp.dialogs import Dialog
from leapp.dialogs.components import BooleanComponent
from leapp.models import Authselect, AuthselectDecision
from leapp.reporting import create_report, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag

resources = [
    reporting.RelatedResource('package', 'authselect'),
    reporting.RelatedResource('package', 'authconfig'),
    reporting.RelatedResource('file', '/etc/nsswitch.conf')
]


class AuthselectCheck(Actor):
    """
    Confirm suggested authselect call from AuthselectScanner.

    AuthselectScanner produces an Authselect model that contains changes
    that are suggested based on current configuration. This actor will
    ask administrator for confirmation and will report the result.
    """

    name = 'authselect_check'
    consumes = (Authselect,)
    produces = (AuthselectDecision, Report,)
    tags = (IPUWorkflowTag, ChecksPhaseTag)
    dialogs = (
        Dialog(
            scope='authselect_check',
            reason='Confirmation',
            components=(
                BooleanComponent(
                    key='confirm',
                    label='Configure PAM and nsswitch.conf with the following '
                          'authselect call?',
                    default=True,
                    description='If yes, suggested authselect profile will '
                                'be applied on your system to generate '
                                'PAM and nsswitch.conf configuration. '
                                'If no, current configuration will be kept '
                                'intact.',
                    reason='There is a new tool called authselect in RHEL8 '
                           'that replaced authconfig which is used to manage '
                           'authentication (PAM) and identity (nsswitch.conf) '
                           'sources. It is recommended to switch to this tool.'
                ),
            )
        ),
    )

    def process(self):
        model = next(self.consume(Authselect))

        # If there is no equivalent authselect profile we will not touch
        # the current configuration. Therefore there is no need for
        # confirmation.
        if model.profile is None:
            self.produce_current_configuration(model)
            return

        command = 'authselect select {0} {1} --force'.format(
            model.profile,
            ' '.join(model.features)
        )

        # We do not need admin confirmation if the current
        # configuration was generated with authconfig.
        if not model.confirm:
            self.produce_authconfig_configuration(model, command)
            return

        # Authselect profile is available but we require confirmation.
        confirmed = self.get_confirmation(model, command)
        if confirmed is not None:
            # A user has made his choice
            self.produce_suggested_configuration(model, confirmed, command)

    def get_confirmation(self, model, command):
        dialog = self.dialogs[0]

        dialog.components[0].label += " {}".format(command)

        return self.get_answers(dialog).get('confirm')

    def produce_authconfig_configuration(self, model, command):
        self.produce(
            AuthselectDecision(
                confirmed=True
            )
        )

        create_report([
            reporting.Title(
                'Authselect will be used to configure PAM and nsswitch.conf.'
            ),
            reporting.Summary(
                'There is a new tool called authselect in RHEL8 that '
                'replaced authconfig. The upgrade process detected '
                'that authconfig was used to generate current '
                'configuration and it will automatically convert it '
                'to authselect. Authselect call is: {}. The process will '
                'also enable "oddjobd" systemd service on startup'.format(command)
            ),
            reporting.Groups([
                reporting.Groups.AUTHENTICATION,
                reporting.Groups.SECURITY,
                reporting.Groups.TOOLS
            ])
        ] + resources)

    def produce_current_configuration(self, model):
        self.produce(
            AuthselectDecision(
                confirmed=False
            )
        )

        create_report([
            reporting.Title(
                'Current PAM and nsswitch.conf configuration will be kept.'
            ),
            reporting.Summary(
                'There is a new tool called authselect in RHEL8 that '
                'replaced authconfig. The upgrade process was unable '
                'to find an authselect profile that would be equivalent '
                'to your current configuration. Therefore your '
                'configuration will be left intact.'
            ),
            reporting.Groups([
                reporting.Groups.AUTHENTICATION,
                reporting.Groups.SECURITY,
                reporting.Groups.TOOLS
            ]),
            reporting.Severity(reporting.Severity.INFO)
        ] + resources)

    def produce_suggested_configuration(self, model, confirmed, command):
        self.produce(
            AuthselectDecision(
                confirmed=confirmed
            )
        )
        if confirmed:
            create_report([
                reporting.Title(
                    'Authselect will be used to configure PAM and nsswitch.conf.'
                ),
                reporting.Summary(
                    'There is a new tool called authselect in RHEL8 that '
                    'replaced authconfig. The upgrade process suggested '
                    'an authselect profile that is similar to your '
                    'current configuration and your system will be switched '
                    'to this profile. Authselect call is: {}. The process will '
                    'also enable "oddjobd" systemd service on startup'.format(command)
                ),
                reporting.Groups([
                    reporting.Groups.AUTHENTICATION,
                    reporting.Groups.SECURITY,
                    reporting.Groups.TOOLS
                ])
            ] + resources)

        else:
            create_report([
                reporting.Title(
                    'Current PAM and nsswitch.conf configuration will be kept.'
                ),
                reporting.Summary(
                    'There is a new tool called authselect in RHEL8 that '
                    'replaced authconfig. The upgrade process suggested '
                    'an authselect profile that is similar to your '
                    'current configuration. However this suggestion was '
                    'refused therefore existing configuration will be kept '
                    'intact.',
                ),
                reporting.Groups([
                    reporting.Groups.AUTHENTICATION,
                    reporting.Groups.SECURITY,
                    reporting.Groups.TOOLS
                ]),
                reporting.Remediation(commands=[[command]]),
                reporting.Severity(reporting.Severity.MEDIUM)
            ] + resources)



import errno
import os

from leapp.libraries.common import multipathutil
from leapp.libraries.common.rpms import has_package
from leapp.libraries.stdlib import api
from leapp.models import (
    CopyFile,
    DistributionSignedRPM,
    MultipathConfFacts8to9,
    MultipathConfig8to9,
    TargetUserSpaceUpgradeTasks
)

_regexes = ('vendor', 'product', 'revision', 'product_blacklist', 'devnode',
            'wwid', 'property', 'protocol')


def _parse_config(path):
    contents = multipathutil.read_config(path)
    if contents is None:
        return None
    conf = MultipathConfig8to9(pathname=path)
    section = None
    in_subsection = False
    for line in contents.split('\n'):
        try:
            data = multipathutil.LineData(line, section, in_subsection)
        except ValueError:
            continue
        if data.type == data.TYPE_BLANK:
            continue
        if data.type == data.TYPE_SECTION_END:
            if in_subsection:
                in_subsection = False
            elif section:
                section = None
            continue
        if data.type == data.TYPE_SECTION_START:
            if not section:
                section = data.section
            elif not in_subsection:
                in_subsection = True
            continue
        if data.type != data.TYPE_OPTION:
            continue
        if section == 'defaults':
            if data.option == 'enable_foreign':
                conf.enable_foreign_exists = True
            elif data.option == 'allow_usb_devices':
                conf.allow_usb_exists = True
            elif data.option == 'config_dir':
                conf.config_dir = data.value
        if data.option in _regexes and data.value == '*':
            conf.invalid_regexes_exist = True
    return conf


def _parse_config_dir(config_dir):
    res = []
    try:
        for config_file in sorted(os.listdir(config_dir)):
            path = os.path.join(config_dir, config_file)
            if not path.endswith('.conf'):
                continue
            conf = _parse_config(path)
            if conf:
                res.append(conf)
    except OSError as e:
        if e.errno == errno.ENOENT:
            api.current_logger().debug('Multipath conf directory ' +
                                       '"{}" doesn\'t exist'.format(config_dir))
        else:
            api.current_logger().warning('Failed to read multipath config ' +
                                         'directory ' +
                                         '"{}": {}'.format(config_dir, e))
    return res


def is_processable():
    res = has_package(DistributionSignedRPM, 'device-mapper-multipath')
    if not res:
        api.current_logger().debug('device-mapper-multipath is not installed.')
    return res


def get_multipath_conf_facts(config_file='/etc/multipath.conf'):
    res_configs = []
    conf = _parse_config(config_file)
    if not conf:
        return None
    res_configs.append(conf)
    if conf.config_dir:
        res_configs.extend(_parse_config_dir(conf.config_dir))
    else:
        res_configs.extend(_parse_config_dir('/etc/multipath/conf.d'))
    return MultipathConfFacts8to9(configs=res_configs)


def produce_copy_to_target_task():
    """
    Produce task to copy files into the target userspace

    The multipath configuration files are needed when the upgrade init ramdisk
    is generated to ensure we are able to boot into the upgrade environment
    and start the upgrade process itself. By this msg it's told that these
    files/dirs will be available when the upgrade init ramdisk is generated.

    See TargetUserSpaceUpgradeTasks and UpgradeInitramfsTasks for more info.
    """
    # TODO(pstodulk): move the function to the multipathconfcheck actor
    # and get rid of the hardcoded stuff.
    # - The current behaviour looks from the user POV same as before this
    # * commit. I am going to keep the proper fix for additional PR as we do
    # * not want to make the current PR even more complex than now and the solution
    # * is not so trivial.
    # - As well, I am missing some information around xDR devices, which are
    # * possibly not handled correctly (maybe missing some executables?..)
    # * Update: practically we do not have enough info about xDR drivers, but
    # * discussed with Ben Marzinski, as the multipath dracut module includes
    # * the xDR utils stuff, we should handle it in the same way.
    # * See xdrgetuid, xdrgetinfo (these two utils are now missing in our initramfs)
    copy_files = []
    for fname in ['/etc/multipath.conf', '/etc/multipath', '/etc/xdrdevices.conf']:
        if os.path.exists(fname):
            copy_files.append(CopyFile(src=fname))

    if copy_files:
        api.produce(TargetUserSpaceUpgradeTasks(copy_files=copy_files))



from leapp.libraries.common import grub
from leapp.libraries.common.config import architecture
from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import DefaultGrubInfo, FirmwareFacts

GRUB_CFG_PATH = '/boot/grub2/grub.cfg'


def _is_grub_cfg_bls():
    # We do not want to use info from the GrubCfgBios model as it may not be valid
    # after the RPM upgrade transaction.

    with open(GRUB_CFG_PATH, 'r') as fo:
        grub_cfg = fo.read()
    if 'insmod blscfg' in grub_cfg:
        return True
    return False


def process():
    if not architecture.matches_architecture(architecture.ARCH_PPC64LE):
        return
    default_grub_msg = next(api.consume(DefaultGrubInfo), None)
    ff = next(api.consume(FirmwareFacts), None)
    if ff and ff.ppc64le_opal:
        return
    if (
        not _is_grub_cfg_bls() and
        grub.is_blscfg_enabled_in_defaultgrub(default_grub_msg)
    ):
        try:
            run(['grub2-mkconfig', '-o', GRUB_CFG_PATH])
        except CalledProcessError as e:
            api.current_logger().error(
                'Command grub2-mkconfig -o {} failed: {}'.format(GRUB_CFG_PATH, e)
            )



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class EnvVar(Model):
    topic = SystemInfoTopic

    name = fields.String()
    """Name of the environment variable."""

    value = fields.String()
    """Value of the environment variable."""


class OSRelease(Model):
    topic = SystemInfoTopic

    release_id = fields.String()
    name = fields.String()
    pretty_name = fields.String()
    version = fields.String()
    version_id = fields.String()
    variant = fields.Nullable(fields.String())  # isn't specified on some systems
    variant_id = fields.Nullable(fields.String())  # same as above


class Version(Model):
    topic = SystemInfoTopic

    source = fields.String()
    """Version of the source (current) system. E.g.: '7.8'."""

    target = fields.String()
    """Version of the target system. E.g. '8.2.'."""


class IPUConfig(Model):
    """
    IPU workflow configuration model
    """
    topic = SystemInfoTopic

    leapp_env_vars = fields.List(fields.Model(EnvVar), default=[])
    """Environment variables related to the leapp."""

    os_release = fields.Model(OSRelease)
    """Data about the OS get from /etc/os-release."""

    version = fields.Model(Version)
    """Version of the current (source) system and expected target system."""

    architecture = fields.String()
    """Architecture of the system. E.g.: 'x86_64'."""

    kernel = fields.String()
    """
    Originally booted kernel when on the source system.
    """

    flavour = fields.StringEnum(('default', 'saphana'), default='default')
    """Flavour of the upgrade - Used to influence changes in supported source/target release"""



from leapp import reporting
from leapp.libraries.actor import migratentp
from leapp.libraries.common.testutils import create_report_mocked


class extract_tgz64_mocked(object):
    def __init__(self):
        self.called = 0
        self.s = None

    def __call__(self, s):
        self.called += 1
        self.s = s


class enable_service_mocked(object):
    def __init__(self):
        self.called = 0
        self.names = []

    def __call__(self, name):
        self.called += 1
        self.names.append(name)


class write_file_mocked(object):
    def __init__(self):
        self.called = 0
        self.name = None
        self.content = None

    def __call__(self, name, content):
        self.called += 1
        self.name = name
        self.content = content


class ntp2chrony_mocked(object):
    def __init__(self, lines):
        self.called = 0
        self.ignored_lines = lines
        self.args = None

    def __call__(self, *args):
        self.called += 1
        self.args = args
        return True, self.ignored_lines * ['a line']


def test_migration(monkeypatch):
    for ntp_services, chrony_services, ignored_lines in [
                ([], [], 0),
                (['ntpd'], ['chronyd'], 0),
                (['ntpdate'], ['chronyd'], 1),
                (['ntp-wait'], ['chrony-wait'], 0),
                (['ntpd', 'ntpdate', 'ntp-wait'], ['chronyd', 'chronyd', 'chrony-wait'], 1),
            ]:
        monkeypatch.setattr(migratentp, 'extract_tgz64', extract_tgz64_mocked())
        monkeypatch.setattr(migratentp, 'enable_service', enable_service_mocked())
        monkeypatch.setattr(migratentp, 'write_file', write_file_mocked())
        monkeypatch.setattr(migratentp, 'ntp2chrony', ntp2chrony_mocked(ignored_lines))

        migratentp.migrate_ntp(ntp_services, 'abcdef')

        if ntp_services:
            assert migratentp.extract_tgz64.called == 1
            assert migratentp.extract_tgz64.s == 'abcdef'
            assert migratentp.enable_service.called == len(chrony_services)
            assert migratentp.enable_service.names == chrony_services
            assert migratentp.write_file.called == (0 if 'ntpd' in ntp_services else 1)
            if migratentp.write_file.called:
                assert migratentp.write_file.name == '/etc/ntp.conf.nosources'
                assert 'without ntp configuration' in migratentp.write_file.content
            assert migratentp.ntp2chrony.called == 1
            assert migratentp.ntp2chrony.args == (
                    '/',
                    '/etc/ntp.conf' if 'ntpd' in ntp_services else '/etc/ntp.conf.nosources',
                    '/etc/ntp/step-tickers' if 'ntpdate' in ntp_services else '')
        else:
            assert migratentp.extract_tgz64.called == 0
            assert migratentp.enable_service.called == 0
            assert migratentp.write_file.called == 0
            assert migratentp.ntp2chrony.called == 0



import os

from leapp.libraries.stdlib import api
from leapp.models import NISConfig

PACKAGES_NAMES = ('ypserv', 'ypbind')
YPBIND_CONF_FILE = '/etc/yp.conf'
YPSERV_DIR_PATH = '/var/yp'
YPSERV_DEFAULT_FILES = ('binding', 'Makefile', 'nicknames')


class NISScanLibrary:
    """
    Helper library for NISScan actor.
    """

    def client_has_non_default_configuration(self):
        """
        Check for any significant ypbind configuration lines in .conf file.
        """
        if not os.path.isfile(YPBIND_CONF_FILE):
            return False

        # Filter whitespaces and empty lines
        with open(YPBIND_CONF_FILE) as f:
            lines = [line.strip() for line in f.readlines() if line.strip()]

        for line in lines:
            # Checks for any valid configuration entry
            if not line.startswith('#'):
                return True
        return False

    def server_has_non_default_configuration(self):
        """
        Check for any additional (not default) files in ypserv DIR.
        """
        if not os.path.isdir(YPSERV_DIR_PATH):
            return False

        return any(f not in YPSERV_DEFAULT_FILES for f in os.listdir(YPSERV_DIR_PATH))

    def process(self):
        """
        Check NIS pkgs configuration for the following options:

        - yp.conf custom configuration
        - /var/yp not default entry
        """
        pkgs = []

        if self.server_has_non_default_configuration():
            pkgs.append('ypserv')

        if self.client_has_non_default_configuration():
            pkgs.append('ypbind')

        api.produce(NISConfig(nis_not_default_conf=pkgs))



from collections import namedtuple
from functools import partial

import pytest

from leapp import reporting
from leapp.libraries.actor.checkmountoptions import check_mount_options
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked
from leapp.libraries.stdlib import api
from leapp.models import FstabEntry, MountEntry, StorageInfo


@pytest.mark.parametrize(
    ('fstab_entries', 'mounts', 'should_inhibit'),
    [
        (
            (('/var', 'default'), ),
            (('/var', 'default'), ),
            False
        ),
        (
            (('/var', 'default'), ('/var/lib', 'default'), ),
            (('/var', 'default'), ('/var/lib', 'default'), ),
            False
        ),
        (
            (('/var', 'default'), ('/var/lib/leapp', 'noexec')),
            (('/var', 'default'), ('/var/lib/leapp', 'noexec')),
            True
        ),
        (
            (('/var', 'defaults'), ('/var/lib', 'noexec')),
            (('/var', 'noexec'), ('/var/lib', 'noexec')),
            True
        ),
        (
            (('/var', 'noexec'), ('/var/lib', 'defaults')),
            (('/var', 'noexec'), ('/var/lib', 'noexec')),
            True
        ),
    ]
)
def test_var_mounted_with_noexec_is_detected(monkeypatch, fstab_entries, mounts, should_inhibit):
    mounts = [
        MountEntry(name='/dev/sdaX', tp='ext4', mount=mountpoint, options=options) for mountpoint, options in mounts
    ]

    fstab_entries = [
        FstabEntry(fs_spec='', fs_file=mountpoint, fs_vfstype='',
                   fs_mntops=opts, fs_freq='0', fs_passno='0') for mountpoint, opts in fstab_entries
    ]

    storage_info = StorageInfo(mount=mounts, fstab=fstab_entries)

    created_reports = create_report_mocked()
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=[storage_info]))
    monkeypatch.setattr(reporting, 'create_report', created_reports)

    check_mount_options()

    assert bool(created_reports.called) == should_inhibit



import errno
import os

from leapp.libraries.actor import spamassassinconfigread
from leapp.libraries.common.testutils import make_IOError, make_OSError


class MockFileOperations(object):
    def __init__(self):
        self.files = {}
        self.files_read = {}
        self.read_called = 0

    def _increment_read_counters(self, path):
        self.read_called += 1
        self.files_read.setdefault(path, 0)
        self.files_read[path] += 1

    def read(self, path):
        self._increment_read_counters(path)
        try:
            return self.files[path]
        except KeyError:
            raise make_IOError(errno.ENOENT)


class MockListDir(object):
    def __init__(self, path=None, file_names=None):
        self.path = None if path is None else os.path.normpath(path)
        self.file_names = file_names
        self.error = False

    def listdir(self, path):
        if os.path.normpath(path) == self.path:
            return self.file_names
        self.error = True
        raise make_OSError(errno.ENOENT)


def test_get_spamassassin_facts():
    spamc_path = '/etc/mail/spamassassin/spamc.conf'
    spamd_path = '/etc/sysconfig/spamassassin'
    fileops = MockFileOperations()
    fileops.files[spamc_path] = '--ssl sslv3'
    fileops.files[spamd_path] = 'SPAMDOPTIONS="--ssl-version tlsv1"'
    listdir = MockListDir(path='/etc/systemd/system', file_names=['spamassassin.service'])

    facts = spamassassinconfigread.get_spamassassin_facts(read_func=fileops.read, listdir=listdir.listdir)

    assert len(fileops.files_read) == 2
    assert spamc_path in fileops.files_read
    assert spamd_path in fileops.files_read
    assert facts.spamc_ssl_argument == 'sslv3'
    assert facts.spamd_ssl_version == 'tlsv1'
    assert facts.service_overriden is True
    assert not listdir.error


def test_get_spamassassin_facts_nonexistent_config():
    spamc_path = '/etc/mail/spamassassin/spamc.conf'
    spamd_path = '/etc/sysconfig/spamassassin'
    fileops = MockFileOperations()
    listdir = MockListDir(path='/etc/systemd/system', file_names=[])

    facts = spamassassinconfigread.get_spamassassin_facts(read_func=fileops.read, listdir=listdir.listdir)

    assert len(fileops.files_read) == 2
    assert spamc_path in fileops.files_read
    assert spamd_path in fileops.files_read
    assert facts.spamc_ssl_argument is None
    assert facts.spamd_ssl_version is None
    assert facts.service_overriden is False
    assert not listdir.error



from leapp.models import fields, Model
from leapp.topics import SystemFactsTopic


class ConsumedDataAsset(Model):
    """Information about a used data asset."""
    topic = SystemFactsTopic

    filename = fields.String()
    fulltext_name = fields.String()
    docs_url = fields.String()
    docs_title = fields.String()
    provided_data_streams = fields.Nullable(fields.List(fields.String()))



import pytest

from leapp.libraries.actor.config_parser import ParsingError, VsftpdConfigOptionParser, VsftpdConfigParser


def test_VsftpdConfigOptionParser_invalid_syntax():
    parser = VsftpdConfigOptionParser()

    with pytest.raises(ParsingError):
        parser.parse_value('unknown option', 'foo')
    with pytest.raises(ParsingError):
        parser.parse_value('anonymous_enable', 'non-boolean value')
    with pytest.raises(ParsingError):
        parser.parse_value('require_cert', 'non-boolean value')
    with pytest.raises(ParsingError):
        parser.parse_value('anon_mkdir_write_enable', '')
    with pytest.raises(ParsingError):
        parser.parse_value('accept_timeout', 'non-integer value')
    with pytest.raises(ParsingError):
        parser.parse_value('max_per_ip', 'non-integer value')
    with pytest.raises(ParsingError):
        parser.parse_value('listen_port', '')


def test_VsftpdConfigOptionParser_string_option():
    parser = VsftpdConfigOptionParser()

    assert parser.parse_value('secure_chroot_dir', 'foo') == 'foo'
    assert parser.parse_value('user_config_dir', '') == ''
    assert parser.parse_value('dsa_cert_file', 'value with spaces') == 'value with spaces'


def test_VsftpdConfigOptionParser_boolean_option():
    parser = VsftpdConfigOptionParser()

    assert parser.parse_value('background', 'TRUE') is True
    assert parser.parse_value('run_as_launching_user', 'true') is True
    assert parser.parse_value('no_log_lock', 'YES') is True
    assert parser.parse_value('force_local_data_ssl', 'yES') is True
    assert parser.parse_value('ssl_tlsv1_2', '1') is True

    assert parser.parse_value('background', 'FALSE') is False
    assert parser.parse_value('run_as_launching_user', 'false') is False
    assert parser.parse_value('no_log_lock', 'NO') is False
    assert parser.parse_value('force_local_data_ssl', 'No') is False
    assert parser.parse_value('ssl_tlsv1_2', '0') is False


def test_VsftpdConfigOptionParser_integer_option():
    parser = VsftpdConfigOptionParser()

    assert parser.parse_value('connect_timeout', '0') == 0
    assert parser.parse_value('idle_session_timeout', '1') == 1
    assert parser.parse_value('data_connection_timeout', '2') == 2
    assert parser.parse_value('pasv_max_port', '6234') == 6234


def test_VsftpdConfigParser_invalid_syntax():
    with pytest.raises(ParsingError):
        VsftpdConfigParser('unknown_option=foo')
    with pytest.raises(ParsingError):
        VsftpdConfigParser('anonymous_enable=non-boolean')
    with pytest.raises(ParsingError):
        VsftpdConfigParser(' # comment with whitespace before the # character')
    with pytest.raises(ParsingError):
        VsftpdConfigParser('anonymous_enable')

    # Make sure that line num is properly shown
    with pytest.raises(ParsingError) as err:
        VsftpdConfigParser('background=0\n#andthislineisalso=fine\nError on line 3')
    assert "Syntax error on line 3" in str(err.value)


def test_VsftpdConfigParser_empty_config():
    parser = VsftpdConfigParser('')
    assert isinstance(parser.parsed_config, dict)
    assert not parser.parsed_config


def test_VsftpdConfigParser_only_comments():
    parser = VsftpdConfigParser('# foo\n\n#bar\n')
    assert isinstance(parser.parsed_config, dict)
    assert not parser.parsed_config

    parser = VsftpdConfigParser('#anonymous_enable=yes\n')
    assert isinstance(parser.parsed_config, dict)
    assert not parser.parsed_config


def test_VsftpdConfigParser_one_option():
    parser = VsftpdConfigParser('anonymous_enable=yes\n')
    assert len(parser.parsed_config) == 1
    assert parser.parsed_config['anonymous_enable'] is True


def test_VsftpdConfigParser_multiple_options():
    content = '# foo\n\nanonymous_enable=no\nbanned_email_file=/foo/bar\n# bar\nmax_login_fails=3\n'
    parser = VsftpdConfigParser(content)

    assert len(parser.parsed_config) == 3
    assert parser.parsed_config['anonymous_enable'] is False
    assert parser.parsed_config['banned_email_file'] == '/foo/bar'
    assert parser.parsed_config['max_login_fails'] == 3



from leapp.models import fields, Model
from leapp.topics import SystemFactsTopic


class DeviceDriverDeprecationEntry(Model):
    """
    Describes a device or driver deprecated in RHEL
    """
    topic = SystemFactsTopic

    deprecation_announced = fields.String()
    """
    This field should contain a X.Y version in which version
    of RHEL the deprecation of the described driver or device was announced.

    NOTE: The value is not enforced to be in the X.Y format and might be empty.
          As not all data entries do contain this relevant information.
    """

    device_id = fields.String()
    """
    If this entry describes a device, this field will contain an identification
    string, relevant to the device.

    This identification string might contain sets of numbers {1,2,3},
    ranges [1-3] or even sets of ranges {[1-3],[5-9]} or a wildcard * which
    should match any number.

    Examples:
    - Range:            'x86_64:intel:6:[56-57]'
    - Set:              'x86_64:amd:23:{1,17,49}'
    - Set of Ranges:    'x86_64:amd:23:{[2-16],[18-48],[50-255]}'
    - Wildcard:         'x86_64:amd:21:*'
    """

    device_type = fields.StringEnum(choices=['pci', 'cpu'])
    """

    NOTE: Other devices might come later. The following values have been
    described:
    - 'usb'
    - 'video_adapter'
    - 'firewire'
    - 'qemu_machine'
    However they aren't in use yet. Since we wouldn't support it for
    now, we can extend it once we support it and just filter the ones supported,
    by leapp.
    """

    device_name = fields.String()
    """
    Human readable name of the device in question - Could be used to display.
    """

    driver_name = fields.String()
    """
    Name of the kernel driver for the device, or just a plain driver name.
    Might be an empty string.
    """

    available_in_rhel = fields.List(fields.Integer())
    """
    List of major version numbers, in which the device/driver can be found.
    If the target major version number is not in this list, the upgrade must
    be inhibited to avoid problems on boot.
    """
    maintained_in_rhel = fields.List(fields.Integer())
    """
    List of major version numbers, in which the device/driver is maintained. That
    means, that in those versions the device/driver actively gets patches and
    bug fixes.
    If the target major version number is not in this list, but is in the
    available_in_rhel list, a warning should be generated, but the upgrade must
    NOT be inhibited.
    """


class DetectedDeviceOrDriver(DeviceDriverDeprecationEntry):
    """
    DetectedDeviceOrDriver is used for reporting unsupported drivers or devices
    """


class DeviceDriverDeprecationData(Model):
    """
    Contains data related to deprecated devices and drivers in RHEL
    """
    topic = SystemFactsTopic

    entries = fields.List(fields.Model(DeviceDriverDeprecationEntry))
    """
    A list of entries describing deprecated devices and drivers
    """



from leapp.models import XorgDrv, XorgDrvFacts
from leapp.reporting import Report


def test_actor_with_deprecated_driver(current_actor_context):
    for driver in ['RADEON', 'ATI', 'AMDGPU', 'MACH64', 'intel', 'spiceqxl', 'qxl', 'NOUVEAU', 'NV', 'VESA']:
        xorg_drv = [XorgDrv(driver=driver, has_options=False)]

        current_actor_context.feed(XorgDrvFacts(xorg_drivers=xorg_drv))
        current_actor_context.run()
        assert current_actor_context.consume(Report)


def test_actor_without_deprecated_driver(current_actor_context):
    xorg_drv = []

    current_actor_context.feed(XorgDrvFacts(xorg_drivers=xorg_drv))
    current_actor_context.run()
    assert not current_actor_context.consume(Report)



import itertools
import sys

from leapp.cli.commands.config import get_config
from leapp.exceptions import UsageError
from leapp.messaging.answerstore import AnswerStore
from leapp.utils.clicmd import command, command_opt


@command('answer', help='Manage answerfile generation: register persistent user choices for specific dialog sections')
@command_opt('section', action='append', metavar='dialog_sections',
             help='Register answer for a specific section in the answerfile')
@command_opt('add', is_flag=True,
             help='If set sections will be created even if missing in original answerfile')
def answer(args):
    """A command to record user choices to the questions in the answerfile.
       Saves user answer between leapp preupgrade runs.
    """
    cfg = get_config()
    if args.section:
        args.section = list(itertools.chain(*[i.split(',') for i in args.section]))
    else:
        raise UsageError('At least one dialog section must be specified, ex. --section dialog.option=mychoice')
    try:
        sections = [tuple((dialog_option.split('.', 2) + [value]))
                    for dialog_option, value in [s.split('=', 2) for s in args.section]]
    except ValueError:
        raise UsageError("A bad formatted section has been passed. Expected format is dialog.option=mychoice")
    answerfile_path = cfg.get('report', 'answerfile')
    answerstore = AnswerStore()
    answerstore.load(answerfile_path)
    for dialog, option, value in sections:
        answerstore.answer(dialog, option, value)
    not_updated = answerstore.update(answerfile_path, allow_missing=args.add)
    if not_updated:
        sys.stderr.write("WARNING: Only sections found in original userfile can be updated, ignoring {}\n".format(
            ",".join(not_updated)))


def register(base_command):
    """
    Registers `leapp answer`
    """
    base_command.add_sub(answer)



import os

from leapp import reporting
from leapp.actors import Actor
from leapp.models import FirmwareFacts
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class EfiCheckBoot(Actor):
    """
    Adjust EFI boot entry for first reboot
    """

    name = 'efi_check_boot'
    consumes = (FirmwareFacts,)
    produces = (reporting.Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        is_system_efi = False
        has_efibootmgr = os.path.exists('/sbin/efibootmgr')
        for fact in self.consume(FirmwareFacts):
            if fact.firmware == 'efi':
                is_system_efi = True
                break

        if is_system_efi and not has_efibootmgr:
            reporting.create_report([
                reporting.Title('efibootmgr package is required on EFI systems'),
                reporting.Summary(
                    'efibootmgr is required so that we can can set proper boot options in between restarts'
                ),
                reporting.Remediation(commands=[['yum', '-y', 'install', 'efibootmgr']]),
                reporting.RelatedResource('package', 'efibootmgr'),
                reporting.Groups([reporting.Groups.INHIBITOR]),
                reporting.Severity(reporting.Severity.HIGH),
                reporting.Groups([reporting.Groups.BOOT])
            ])



import pytest

from leapp.libraries.common.config import architecture
from leapp.libraries.common.testutils import CurrentActorMocked
from leapp.libraries.stdlib import api


def test_matches_architecture_pass(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(arch=architecture.ARCH_ACCEPTED[0]))
    assert architecture.matches_architecture(*architecture.ARCH_ACCEPTED)


def test_matches_architecture_fail(monkeypatch):
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(arch=architecture.ARCH_ACCEPTED[0]))
    assert not architecture.matches_architecture()
    assert not architecture.matches_architecture(*architecture.ARCH_ACCEPTED[1:])


def test_matches_architecture_wrong_args():
    with pytest.raises(TypeError):
        architecture.matches_architecture('aarch64', 1)



import os

import pytest

from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import SELinuxModule, SELinuxModules
from leapp.reporting import Report
from leapp.snactor.fixture import current_actor_context

TEST_MODULES = [
    ['400', 'mock1'],
    ['99', 'mock1'],
    ['300', 'mock1'],
    ['400', 'mock2'],
    ['999', 'mock3'],
]

TEST_TEMPLATES = [
    ['200', 'base_container']
]

SEMANAGE_COMMANDS = [
    ['fcontext', '-t', 'httpd_sys_content_t', '"/web(/.*)?"'],
    ['fcontext', '-t', 'cgdcbxd_var_run_t', '"/ganesha(/.*)?"'],
    ['fcontext', '-t', 'mock_file_type_t', '"/mock_directory(/.*)?"'],
    ['port', '-t', 'http_port_t', '-p', 'udp', '81'],
    ['permissive', 'abrt_t']
]

testmoduledir = 'tests/mock_modules/'


def _run_cmd(cmd, logmsg='', split=False):
    try:
        return run(cmd, split=split).get('stdout', '')
    except CalledProcessError as e:
        if logmsg:
            api.current_logger().warning('{}: {}'.format(logmsg, e.stderr))
    return None


@pytest.fixture(scope='module')
def semodule_lfull_initial():
    yield _run_cmd(['semodule', '-lfull'], logmsg='Error listing SELinux customizations')


@pytest.fixture(scope='module')
def semanage_export_initial():
    yield _run_cmd(['semanage', 'export'], logmsg='Error listing SELinux customizations')


@pytest.fixture(scope='function')
def destructive_selinux_env():
    tests_dir = os.path.join(os.getenv('PYTEST_CURRENT_TEST').rsplit(os.path.sep, 2)[0], testmoduledir)

    # try to install compatibility module - needed on newer systems - failure to install is expected on rhel 7
    _run_cmd(['semodule', '-X', '100', '-i', os.path.join(tests_dir, 'compat.cil')])

    semodule_command = ['semodule']
    for priority, module in TEST_MODULES + TEST_TEMPLATES:
        semodule_command.extend(['-X', priority, '-i', os.path.join(tests_dir, module + '.cil')])
    _run_cmd(semodule_command, logmsg='Error installing mock modules')

    for command in SEMANAGE_COMMANDS:
        _run_cmd(['semanage', command[0], '-a'] + command[1:], logmsg='Error applying selinux customizations')

    yield

    for command in SEMANAGE_COMMANDS:
        _run_cmd(['semanage', command[0], '-d'] + command[1:])

    semodule_command = ['semodule']
    for priority, module in reversed(TEST_MODULES + TEST_TEMPLATES +
                                     [['400', 'permissive_abrt_t'], ['100', 'compat']]):
        semodule_command.extend(['-X', priority, '-r', module])
    _run_cmd(semodule_command)


@pytest.mark.skipif(os.getenv('DESTRUCTIVE_TESTING', False) in [False, '0'],
                    reason='Test disabled by default because it would modify the system')
def test_SELinuxPrepare(current_actor_context, semodule_lfull_initial, semanage_export_initial,
                        destructive_selinux_env):
    before_test = []
    for cmd in (['semodule', '-lfull'], ['semanage', 'export']):
        res = _run_cmd(cmd, 'Error listing SELinux customizations')
        before_test.append(res)
        # XXX still not sure about logging in tests
        api.current_logger().info('Before test: {}'.format(res))
    # Make sure that initial semodule/semanage commands don't match before tests ones
    assert before_test != [semodule_lfull_initial, semanage_export_initial]

    semodule_list = [SELinuxModule(name=module, priority=int(prio), content='', removed=[])
                     for (prio, module) in TEST_MODULES + [['400', 'permissive_abrt_t'], ['100', 'compat']]]

    template_list = [SELinuxModule(name=module, priority=int(prio), content='', removed=[])
                     for (prio, module) in TEST_TEMPLATES]

    current_actor_context.feed(SELinuxModules(modules=semodule_list, templates=template_list))

    current_actor_context.run()

    # check if all given modules and local customizations where removed
    semodule_res = _run_cmd(['semodule', '-lfull'], 'Error listing SELinux modules')
    assert semodule_lfull_initial == semodule_res
    semanage_res = _run_cmd(['semanage', 'export'], 'Error listing SELinux customizations')
    assert semanage_export_initial == semanage_res



import logging
import sys

from leapp.repository.scan import find_and_scan_repositories


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO, filename='/dev/null')
    logger = logging.getLogger('run_pytest.py')

    BASE_REPO = 'repos'
    repos = find_and_scan_repositories(BASE_REPO, include_locals=True)
    repos.load()

    print(' '.join(repos.libraries))



import leapp.libraries.actor.checkmicroarchitecture as checkmicroarchitecture
from leapp.actors import Actor
from leapp.models import CPUInfo
from leapp.reporting import Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckMicroarchitecture(Actor):
    """
    Inhibit if RHEL9 microarchitecture requirements are not satisfied


    As per `x86-64-ABI`_ In addition to the AMD64 baseline architecture, several
    micro-architecture levels implemented by later CPU modules have been
    defined, starting at level ``x86-64-v2``. The levels are cumulative in the
    sense that features from previous levels are implicitly included in later
    levels.

    RHEL9 has a higher CPU requirement than older versions, it now requires a
    CPU compatible with ``x86-64-v2`` instruction set or higher.

    .. table:: Required CPU features by microarchitecure level with a
               corresponding flag as shown by ``lscpu``.

        +------------+-------------+--------------------+
        | Version    | CPU Feature | flag (lscpu)       |
        +============+=============+====================+
        | (baseline) | CMOV        | cmov               |
        |            | CX8         | cx8                |
        |            | FPU         | fpu                |
        |            | FXSR        | fxsr               |
        |            | MMX         | mmx                |
        |            | OSFXSR      | (common with FXSR) |
        |            | SCE         | syscall            |
        |            | SSE         | sse                |
        |            | SSE2        | sse2               |
        +------------+-------------+--------------------+
        | x86-64-v2  | CMPXCHG16B  | cx16               |
        |            | LAHF-SAHF   | lahf_lm            |
        |            | POPCNT      | popcnt             |
        |            | SSE3        | pni                |
        |            | SSE4_1      | sse4_1             |
        |            | SSE4_2      | sse4_2             |
        |            | SSSE3       | ssse3              |
        +------------+-------------+--------------------+
        | ...        |             |                    |
        +------------+-------------+--------------------+

    Note: To get the corresponding flag for the CPU feature consult the file
    ``/arch/x86/include/asm/cpufeatures.h`` in the linux kernel.


    .. _x86-64-ABI: https://gitlab.com/x86-psABIs/x86-64-ABI.git

    """

    name = 'check_microarchitecture'
    consumes = (CPUInfo,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag,)

    def process(self):
        checkmicroarchitecture.process()



from leapp.actors import Actor
from leapp.libraries.actor.removeoldpammodulesapply import comment_modules, read_file
from leapp.libraries.common.pam import PAM
from leapp.models import RemovedPAMModules
from leapp.tags import IPUWorkflowTag, PreparationPhaseTag


class RemoveOldPAMModulesApply(Actor):
    """
    Remove old PAM modules that are no longer available in RHEL-8 from
    PAM configuration to avoid system lock out.
    """

    name = 'removed_pam_modules_apply'
    consumes = (RemovedPAMModules,)
    produces = ()
    tags = (IPUWorkflowTag, PreparationPhaseTag)

    def process(self):
        for model in self.consume(RemovedPAMModules):
            for path in PAM.files:
                content = read_file(path)
                if not content:  # Nothing to do if no content?
                    continue

                with open(path, 'w') as f:
                    f.write(comment_modules(model.modules, content))
            break



from leapp.actors import Actor
from leapp.libraries.actor import rocescanner
from leapp.models import RoceDetected
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class RoCEScanner(Actor):
    """
    Detect active RoCE NICs on IBM Z machines.

    Detect whether RoCE is configured on the system and produce
    the RoceDetected message with active RoCE NICs - if any exists.
    The active connections are scanned using NetworkManager (`nmcli`) as
    RoCE is supposed to be configured via NetworkManager since
    RHEL 8; see:
        https://www.ibm.com/docs/en/linux-on-systems?topic=guide-add-additional-roce-interface

    The scan is performed only on IBM Z machines.
    """

    name = 'roce_scanner'
    consumes = ()
    produces = (RoceDetected,)
    tags = (FactsPhaseTag, IPUWorkflowTag)

    def process(self):
        rocescanner.process()



import pytest

from leapp import reporting
from leapp.libraries.actor.checkpersistentmounts import check_persistent_mounts
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked
from leapp.libraries.stdlib import api
from leapp.models import FstabEntry, MountEntry, StorageInfo

MOUNT_ENTRY = MountEntry(name='/dev/sdaX', tp='ext4', mount='/var/lib/leapp', options='defaults')

FSTAB_ENTRY = FstabEntry(fs_spec='', fs_file='/var/lib/leapp', fs_vfstype='',
                         fs_mntops='defaults', fs_freq='0', fs_passno='0')


@pytest.mark.parametrize(
    ('storage_info', 'should_inhibit'),
    [
        (
            StorageInfo(mount=[MOUNT_ENTRY], fstab=[]),
            True
        ),
        (
            StorageInfo(mount=[], fstab=[FSTAB_ENTRY]),
            False
        ),
        (
            StorageInfo(mount=[MOUNT_ENTRY], fstab=[FSTAB_ENTRY]),
            False
        ),
    ]
)
def test_var_lib_leapp_non_persistent_is_detected(monkeypatch, storage_info, should_inhibit):

    created_reports = create_report_mocked()
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=[storage_info]))
    monkeypatch.setattr(reporting, 'create_report', created_reports)

    check_persistent_mounts()

    assert bool(created_reports.called) == should_inhibit



import os

import pytest

from leapp.libraries.actor.scanfilesfortargetuserspace import scan_files_to_copy
from leapp.libraries.common.testutils import produce_mocked
from leapp.libraries.stdlib import api


@pytest.fixture
def isfile_default_config():
    config = {
        '/etc/hosts': True
    }
    return config


def do_files_to_copy_contain_entry_with_src(files_to_copy, src):
    """Searches the files to be copied for an entry with src field that matches the given src."""
    is_present = False
    for file_to_copy in files_to_copy:
        if file_to_copy.src == src:
            is_present = True
            break
    return is_present


def make_mocked_isfile(configuration):
    """
    Creates mocked isfile function that returns values according the given configuration.

    The created function raises :class:`ValueError` should the unit under test try to "isfile"
    a path that is not present in the configuration.

    One global mocked function with configuration is error prone as individual test would
    have to return the configuration into the original state after execution.
    """

    def mocked_isfile(path):
        if path in configuration:
            return configuration[path]
        error_msg = 'The actor tried to isfile a path that it should not. (path `{0}`)'
        raise ValueError(error_msg.format(path))
    return mocked_isfile


def test_etc_hosts_present(monkeypatch, isfile_default_config):
    """Tests whether /etc/hosts is identified as "to be copied" into target userspace when it is present."""
    mocked_isfile = make_mocked_isfile(isfile_default_config)
    actor_produces = produce_mocked()
    monkeypatch.setattr(os.path, 'isfile', mocked_isfile)
    monkeypatch.setattr(api, 'produce', actor_produces)

    scan_files_to_copy()

    fail_msg = 'Produced unexpected number of messages.'
    assert len(actor_produces.model_instances) == 1, fail_msg

    preupgrade_task_msg = actor_produces.model_instances[0]

    fail_msg = 'Didn\'t identify any files to copy into target userspace (at least /etc/hosts should be).'
    assert preupgrade_task_msg.copy_files, fail_msg

    should_copy_hostsfile = do_files_to_copy_contain_entry_with_src(preupgrade_task_msg.copy_files, '/etc/hosts')
    assert should_copy_hostsfile, 'Failed to identify /etc/hosts as a file to be copied into target userspace.'

    fail_msg = 'Produced message contains rpms to be installed, however only copy files field should be populated.'
    assert not preupgrade_task_msg.install_rpms, fail_msg


def test_etc_hosts_missing(monkeypatch, isfile_default_config):
    """Tests whether /etc/hosts is not identified as "to be copied" into target userspace when it is missing."""
    isfile_default_config['/etc/hosts'] = False  # The file is not present or is a directory (-> should not be copied)
    mocked_isfile = make_mocked_isfile(isfile_default_config)
    actor_produces = produce_mocked()

    monkeypatch.setattr(os.path, 'isfile', mocked_isfile)
    monkeypatch.setattr(api, 'produce', actor_produces)

    scan_files_to_copy()

    assert len(actor_produces.model_instances) == 1, 'Produced unexpected number of messages.'

    preupgrade_task_msg = actor_produces.model_instances[0]
    should_copy_hostsfile = do_files_to_copy_contain_entry_with_src(preupgrade_task_msg.copy_files, '/etc/hosts')
    assert not should_copy_hostsfile, 'Identified /etc/hosts as a file to be copied even if it doesn\'t exists'

    fail_msg = 'Produced message contains rpms to be installed, however only copy files field should be populated.'
    assert not preupgrade_task_msg.install_rpms, fail_msg



import pytest

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.actor.cupsscanner import find_features

message = 'Checking if CUPS configuration contains removed features.'

testdata = (
    (
        ['ble'],
        {},
        {}
    ),
    (
        ['cups'],
        {
            '/etc/cups/cupsd.conf': '',
            '/etc/cups/cups-files.conf': ''
        },
        {
            'debug': message
        }
    ),
    (
        ['cups'],
        {
            '/etc/cups/cupsd.conf': 'Include smth\n',
            '/etc/cups/cups-files.conf': '',
            'smth': ''
        },
        {
            'include': True,
            'digest': False,
            'interface': False,
            'env': False,
            'certkey': False,
            'printcap': False,
            'included_files': ['/etc/cups/cupsd.conf', 'smth'],
            'debug': message
        }
    ),
    (
        ['cups'],
        {
            '/etc/cups/cupsd.conf': 'Include smth\n',
            '/etc/cups/cups-files.conf': '',
        },
        {
            'debug': message,
            'warn': 'Following included files will not be appended to cupsd.c'
            'onf due attached error:\n   - Error during reading file smth: file not found'
        }
    ),
    (
        ['cups'],
        {
            '/etc/cups/cupsd.conf': 'Include smth\nInclude smb\n',
            '/etc/cups/cups-files.conf': '',
            'smth': '',
            'smb': ''
        },
        {
            'include': True,
            'digest': False,
            'interface': False,
            'env': False,
            'certkey': False,
            'printcap': False,
            'included_files': ['/etc/cups/cupsd.conf', 'smth', 'smb'],
            'debug': message
        }
    ),
    (
        ['cups'],
        {
            '/etc/cups/cupsd.conf': 'Include smth\n',
            '/etc/cups/cups-files.conf': '',
            'smth': 'AuthType Digest\nPassEnv smth\nPrintcapFormat neco\n'
        },
        {
            'include': True,
            'digest': True,
            'interface': False,
            'env': True,
            'certkey': False,
            'printcap': True,
            'included_files': ['/etc/cups/cupsd.conf', 'smth'],
            'debug': message
        }
    ),
    (
        ['cups'],
        {
            '/etc/cups/cupsd.conf': '',
            '/etc/cups/cups-files.conf': 'ServerKey smth.key\n',
            'smth.key': ''
        },
        {
            'include': False,
            'digest': False,
            'interface': False,
            'env': False,
            'certkey': True,
            'printcap': False,
            'included_files': ['/etc/cups/cupsd.conf'],
            'debug': message
        }
    ),
    (
        ['cups'],
        {
            '/etc/cups/cupsd.conf': '',
            '/etc/cups/cups-files.conf': 'ServerCertificate smth.cert\n',
            'smth.cert': ''
        },
        {
            'include': False,
            'digest': False,
            'interface': False,
            'env': False,
            'certkey': True,
            'printcap': False,
            'included_files': ['/etc/cups/cupsd.conf'],
            'debug': message
        }
    ),
    (
        ['cups'],
        {
            '/etc/cups/cupsd.conf': '',
            '/etc/cups/cups-files.conf': 'ServerKey smth.key\n'
                                         'ServerCertificate smth.cert\n',
            'smth.key': '',
            'smth.cert': ''
        },
        {
            'include': False,
            'digest': False,
            'interface': False,
            'env': False,
            'certkey': True,
            'printcap': False,
            'included_files': ['/etc/cups/cupsd.conf'],
            'debug': message
        }
    ),
    (
        ['cups'],
        {
            '/etc/cups/cupsd.conf': 'AuthType Digest\n',
            '/etc/cups/cups-files.conf': '',
        },
        {
            'include': False,
            'digest': True,
            'interface': False,
            'env': False,
            'certkey': False,
            'printcap': False,
            'included_files': ['/etc/cups/cupsd.conf'],
            'debug': message
        }
    ),
    (
        ['cups'],
        {
            '/etc/cups/cupsd.conf': 'DefaultAuthType BasicDigest\n',
            '/etc/cups/cups-files.conf': '',
        },
        {
            'include': False,
            'digest': True,
            'interface': False,
            'env': False,
            'certkey': False,
            'printcap': False,
            'included_files': ['/etc/cups/cupsd.conf'],
            'debug': message
        }
    ),
    (
        ['cups'],
        {
            '/etc/cups/cupsd.conf': 'PassEnv smth\n',
            '/etc/cups/cups-files.conf': '',
        },
        {
            'include': False,
            'digest': False,
            'interface': False,
            'env': True,
            'certkey': False,
            'printcap': False,
            'included_files': ['/etc/cups/cupsd.conf'],
            'debug': message
        }
    ),
    (
        ['cups'],
        {
            '/etc/cups/cupsd.conf': 'SetEnv smth\n',
            '/etc/cups/cups-files.conf': '',
        },
        {
            'include': False,
            'digest': False,
            'interface': False,
            'env': True,
            'certkey': False,
            'printcap': False,
            'included_files': ['/etc/cups/cupsd.conf'],
            'debug': message
        }
    ),
    (
        ['cups'],
        {
            '/etc/cups/cupsd.conf': 'PrintcapFormat smth\n',
            '/etc/cups/cups-files.conf': '',
        },
        {
            'include': False,
            'digest': False,
            'interface': False,
            'env': False,
            'certkey': False,
            'printcap': True,
            'included_files': ['/etc/cups/cupsd.conf'],
            'debug': message
        }
    ),
    (
        ['cups'],
        {
            '/etc/cups/cupsd.conf': '',
            '/etc/cups/cups-files.conf': '',
            '/etc/cups/interfaces': []
        },
        {
            'debug': message
        }
    ),
    (
        ['cups'],
        {
            '/etc/cups/cupsd.conf': '',
            '/etc/cups/cups-files.conf': '',
            '/etc/cups/interfaces': ['smth', 'anything'],
            'smth': '',
            'anything': ''
        },
        {
            'include': False,
            'digest': False,
            'interface': True,
            'env': False,
            'certkey': False,
            'printcap': False,
            'included_files': ['/etc/cups/cupsd.conf'],
            'debug': message
        }
    ),
    (
        ['cups'],
        {
            '/etc/cups/cupsd.conf': 'Include mst\nAuthType Digest\n'
                                    'PassEnv too\nPrintcapFormat poo\n',
            '/etc/cups/cups-files.conf': 'ServerKey my.key\n'
                                         'ServerCertificate my.cert\n',
            '/etc/cups/interfaces': ['smth', 'anything'],
            'smth': '',
            'anything': '',
            'mst': ''
        },
        {
            'include': True,
            'digest': True,
            'interface': True,
            'env': True,
            'certkey': True,
            'printcap': True,
            'included_files': ['/etc/cups/cupsd.conf', 'mst'],
            'debug': message
        }
    )
)


class MockActor(object):
    def __init__(self):
        self.output = {}

    def send_features(self, interface, digest, include, certkey, env,
                      printcap, included_files):
        self.output['interface'] = interface
        self.output['digest'] = digest
        self.output['include'] = include
        self.output['certkey'] = certkey
        self.output['env'] = env
        self.output['printcap'] = printcap
        self.output['included_files'] = included_files


class MockLogger(object):
    def __init__(self):
        self.debugmsg = ''
        self.warnmsg = ''
        self.errormsg = ''

    def debug(self, message):
        self.debugmsg += message

    def error(self, message):
        self.errormsg += message

    def warn(self, message):
        self.warnmsg += message


class MockFileSystem(object):
    def __init__(self, packages, files):
        self.installed_packages = packages
        self.files = files

    def is_installed(self, pkg):
        if pkg in self.installed_packages:
            return True
        return False

    def read(self, path):
        if path in self.files.keys():
            return self.files[path].splitlines(True)
        raise IOError('Error during reading file {} - file'
                      ' not found.'.format(path))

    def path_exists(self, path):
        if path in self.files.keys():
            return True
        return False

    def list_dir(self, path):
        if path in self.files.keys():
            return self.files[path]
        return False


def test_find_features_exception():
    logger = MockLogger()
    system = MockFileSystem(['cups'], {})
    actor = MockActor()

    with pytest.raises(StopActorExecutionError):
        find_features(logger.debug,
                      logger.warn,
                      logger.error,
                      actor.send_features,
                      system.is_installed,
                      system.read,
                      system.path_exists,
                      system.list_dir)


@pytest.mark.parametrize(("packages,files,expected"), testdata)
def test_find_features(packages, files, expected):
    logger = MockLogger()
    system = MockFileSystem(packages, files)
    actor = MockActor()

    find_features(logger.debug,
                  logger.warn,
                  logger.error,
                  actor.send_features,
                  system.is_installed,
                  system.read,
                  system.path_exists,
                  system.list_dir)

    assert actor.output.get('interface', None) == expected.get('interface', None)
    assert actor.output.get('digest', None) == expected.get('digest', None)
    assert actor.output.get('include', None) == expected.get('include', None)
    assert actor.output.get('certkey', None) == expected.get('certkey', None)
    assert actor.output.get('env', None) == expected.get('env', None)
    assert actor.output.get('printcap', None) == expected.get('printcap', None)
    assert actor.output.get('included_files', None) == expected.get('included_files', None)
    assert logger.debugmsg == expected.get('debug', '')
    assert logger.warnmsg == expected.get('warn', '')
    assert logger.errormsg == expected.get('error', '')



from leapp.libraries.actor.opensshpermitrootlogincheck import global_value, semantics_changes
from leapp.models import OpenSshConfig, OpenSshPermitRootLogin


def test_empty_file():
    """ Empty file
    """
    config = OpenSshConfig(
        permit_root_login=[
        ],
        deprecated_directives=[]
    )

    assert semantics_changes(config)
    assert global_value(config, "default") == "default"


def test_globally_enabled():
    """ Configuration file in this format:

        PermitRootLogin yes # explicit
    """
    config = OpenSshConfig(
        permit_root_login=[
            OpenSshPermitRootLogin(
                value='yes',
                in_match=None)
        ],
        deprecated_directives=[]
    )

    assert not semantics_changes(config)
    assert global_value(config, "default") == "yes"


def test_globally_disabled():
    """ Configuration file in this format:

        PermitRootLogin no # explicit
    """
    config = OpenSshConfig(
        permit_root_login=[
            OpenSshPermitRootLogin(
                value='no',
                in_match=None)
        ],
        deprecated_directives=[]
    )

    assert not semantics_changes(config)
    assert global_value(config, "default") == "no"


def test_globally_disabled_password():
    """ Configuration file in this format:

        PermitRootLogin prohibit-password # explicit
    """
    config = OpenSshConfig(
        permit_root_login=[
            OpenSshPermitRootLogin(
                value='prohibit-password',
                in_match=None)
        ],
        deprecated_directives=[]
    )

    assert not semantics_changes(config)
    assert global_value(config, "default") == "prohibit-password"


def test_in_match_disabled():
    """ Configuration file in this format:

        # PermitRootLogin yes # implicit
        Match address 10.10.*
            PermitRootLogin no
    """
    config = OpenSshConfig(
        permit_root_login=[
            OpenSshPermitRootLogin(
                value='no',
                in_match=['address', '10.10.*'])
        ],
        deprecated_directives=[]
    )

    assert semantics_changes(config)
    assert global_value(config, "default") == "default"


def test_in_match_disabled_password():
    """ Configuration file in this format:

        # PermitRootLogin yes # implicit
        Match address 192.168.*
            PermitRootLogin prohibit-password
    """
    config = OpenSshConfig(
        permit_root_login=[
            OpenSshPermitRootLogin(
                value='prohibit-password',
                in_match=['address', '10.10.*'])
        ],
        deprecated_directives=[]
    )

    assert semantics_changes(config)
    assert global_value(config, "default") == "default"


def test_in_match_enabled():
    """ Configuration file in this format:

        # PermitRootLogin yes # implicit
        Match address 192.168.*
            PermitRootLogin yes
    """
    # TODO This is suspicious configuration we should probably handle separately
    config = OpenSshConfig(
        permit_root_login=[
            OpenSshPermitRootLogin(
                value='yes',
                in_match=['address', '192.168.*'])
        ],
        deprecated_directives=[]
    )

    assert not semantics_changes(config)
    assert global_value(config, "default") == "default"


def test_in_match_all_disabled():
    """ Configuration file in this format:

        # PermitRootLogin yes # implicit
        Match all
            PermitRootLogin no
    """
    config = OpenSshConfig(
        permit_root_login=[
            OpenSshPermitRootLogin(
                value='no',
                in_match=['all'])
        ],
        deprecated_directives=[]
    )

    assert not semantics_changes(config)
    assert global_value(config, "default") == "no"


def test_in_match_all_disabled_password():
    """ Configuration file in this format:

        # PermitRootLogin yes # implicit
        Match all
            PermitRootLogin prohibit-password
    """
    config = OpenSshConfig(
        permit_root_login=[
            OpenSshPermitRootLogin(
                value='prohibit-password',
                in_match=['all'])
        ],
        deprecated_directives=[]
    )

    assert not semantics_changes(config)
    assert global_value(config, "default") == "prohibit-password"


def test_in_match_all_enabled():
    """ Configuration file in this format:

        # PermitRootLogin yes # implicit
        Match all
            PermitRootLogin yes
    """
    config = OpenSshConfig(
        permit_root_login=[
            OpenSshPermitRootLogin(
                value='yes',
                in_match=['all'])
        ],
        deprecated_directives=[]
    )

    assert not semantics_changes(config)
    assert global_value(config, "default") == "yes"


def test_in_match_enabled_globally_disabled():
    """ Configuration file in this format:

        PermitRootLogin no # explicit
        Match address 192.*
            PermitRootLogin yes
    """
    config = OpenSshConfig(
        permit_root_login=[
            OpenSshPermitRootLogin(
                value='no',
                in_match=None),
            OpenSshPermitRootLogin(
                value='yes',
                in_match=['address', '192.*'])
        ],
        deprecated_directives=[]
    )

    assert not semantics_changes(config)
    assert global_value(config, "default") == "no"


def test_in_match_disabled_globally_enabled():
    """ Configuration file in this format:

        PermitRootLogin yes # explicit
        Match address 192.*
            PermitRootLogin no
    """
    config = OpenSshConfig(
        permit_root_login=[
            OpenSshPermitRootLogin(
                value='yes',
                in_match=None),
            OpenSshPermitRootLogin(
                value='no',
                in_match=['address', '192.*'])
        ],
        deprecated_directives=[]
    )

    assert not semantics_changes(config)
    assert global_value(config, "default") == "yes"



from leapp.models import fields, Model
from leapp.topics import SystemFactsTopic
from leapp.utils.deprecation import deprecated


class RepositoryData(Model):
    topic = SystemFactsTopic

    repoid = fields.String()
    name = fields.String()
    baseurl = fields.Nullable(fields.String())
    metalink = fields.Nullable(fields.String())
    mirrorlist = fields.Nullable(fields.String())
    enabled = fields.Boolean(default=True)
    additional_fields = fields.Nullable(fields.String())
    proxy = fields.Nullable(fields.String())


class RepositoryFile(Model):
    topic = SystemFactsTopic

    file = fields.String()
    data = fields.List(fields.Model(RepositoryData))


class RepositoriesFacts(Model):
    topic = SystemFactsTopic

    repositories = fields.List(fields.Model(RepositoryFile))


@deprecated(
    since="2020-09-01",
    message=(
        "The model is temporary and not assumed to be used in any "
        "other actors."
    ),
)
class TMPTargetRepositoriesFacts(RepositoriesFacts):
    """Do not consume this model anywhere outside of localreposinhibit.

    The model is temporary and will be removed in close future
    """

    pass



import pytest

from leapp.models import DistributionSignedRPM, Report, RPM
from leapp.snactor.fixture import current_actor_context

RH_PACKAGER = 'Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>'

ballast1 = [
    RPM(name='b1', version='1', release='1', epoch='1', packager=RH_PACKAGER, arch='noarch', pgpsig='s'),
    RPM(name='kernel', version='1', release='1', epoch='1', packager=RH_PACKAGER, arch='noarch', pgpsig='s'),
    RPM(name='b2', version='1', release='1', epoch='1', packager=RH_PACKAGER, arch='noarch', pgpsig='s')
]
ballast2 = [
    RPM(name='b3', version='1', release='1', epoch='1', packager=RH_PACKAGER, arch='noarch', pgpsig='s'),
    RPM(name='kernel', version='1', release='1', epoch='1', packager=RH_PACKAGER, arch='noarch', pgpsig='s'),
    RPM(name='b4', version='1', release='1', epoch='1', packager=RH_PACKAGER, arch='noarch', pgpsig='s')
]
devel_kernels = [
    RPM(name='kernel-devel', version='3.10.0', release='957.27.4.el7',
        epoch='1', packager=RH_PACKAGER, arch='noarch', pgpsig='s'),
    RPM(name='kernel-devel', version='3.10.0', release='957.35.1.el7',
        epoch='1', packager=RH_PACKAGER, arch='noarch', pgpsig='s'),
    RPM(name='kernel-devel', version='3.10.0', release='957.43.1.el7',
        epoch='1', packager=RH_PACKAGER, arch='noarch', pgpsig='s')
]


@pytest.mark.parametrize('n', [0, 1, 2, 3])
def test_process_devel_kernels(current_actor_context, n):
    current_actor_context.feed(DistributionSignedRPM(items=ballast1+devel_kernels[:n]+ballast2))
    current_actor_context.run()
    if n < 2:
        assert not current_actor_context.consume(Report)
    else:
        assert current_actor_context.consume(Report)



from leapp.actors import Actor
from leapp.libraries.actor import checketcreleasever
from leapp.models import PkgManagerInfo, Report, RHUIInfo
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckEtcReleasever(Actor):
    """
    Check releasever info and provide a guidance based on the facts
    """

    name = 'check_etc_releasever'
    consumes = (PkgManagerInfo, RHUIInfo)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        checketcreleasever.process()



from leapp.actors import Actor
from leapp.libraries.actor.checkchrony import check_chrony
from leapp.libraries.common.rpms import has_package
from leapp.models import DistributionSignedRPM
from leapp.reporting import Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckChrony(Actor):
    """
    Check for incompatible changes in chrony configuration.

    Warn that the default chrony configuration in RHEL8 uses the leapsectz
    directive.
    """

    name = 'check_chrony'
    consumes = (DistributionSignedRPM,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        check_chrony(has_package(DistributionSignedRPM, 'chrony'))



import os

from leapp import models
from leapp.libraries.common import rpms
from leapp.libraries.stdlib import api, run

MIN_DISK_SIZE = 2 ** 22  # 4 MiB


def _check_vdo_lvm_managed(device):
    """
    Determines if the specified device (which has already been identified
    as a post-conversion vdo device (at the level of vdo) is managed by lvm.
    """
    command = ['blkid', '--match-token', 'TYPE=LVM2_member', device]
    result = run(command, checked=False)
    exit_code = result['exit_code']
    #     0: Is LVM managed
    #     2: Is not LVM manaaged
    # other: Unexpected
    return exit_code


def _check_vdo_pre_conversion(device):
    """
    Identify if the specified device is either not a vdo device, a
    pre-conversion vdo device or a post-conversion vdo device.
    """
    command = ['/usr/libexec/vdoprepareforlvm', '--check', device]
    result = run(command, checked=False)
    exit_code = result['exit_code']
    #   255: Not a vdo device
    #     0: A post-conversion vdo device
    #     1: A pre-conversion vdo device
    # other: Unexpected
    return exit_code


def _lvm_package_installed():
    return rpms.has_package(models.DistributionSignedRPM, 'lvm2')


def _vdo_package_installed():
    return rpms.has_package(models.DistributionSignedRPM, 'vdo')


def get_info(storage_info):
    pre_conversion_devices = []
    post_conversion_devices = []
    undetermined_conversion_devices = []

    # Only if lvm is installed can there be VDO instances.
    if _lvm_package_installed():
        vdo_package_installed = _vdo_package_installed()

        for lsblk in storage_info.lsblk:
            # NOTE: partitions < MIN_DISK_SIZE cannot be handled by vdo and
            # the check results in unexpected outputs
            if lsblk.tp not in ('disk', 'part') or lsblk.bsize < MIN_DISK_SIZE:
                continue

            if not vdo_package_installed:
                undetermined_conversion_devices.append(
                    models.VdoConversionUndeterminedDevice(name=lsblk.name))
                continue

            # refer to kernel name
            device = '/dev/{0}'.format(lsblk.kname)
            if not os.path.exists(device):
                # NOTE: Corner case. It's hypothetical situation which could possibly
                # happen but we do not know under what circumstances and we do not
                # have time now for investigation. Let's see if someone report it
                # to us so we will have a data :)
                # For now, stay on the safe side and inhibit the upgrade if this
                # happens.
                failure = (
                    'cannot check device {0} (kernel name: {1}): file {2} does not exist'
                    .format(lsblk.name, lsblk.kname, device)
                )
                api.current_logger().warning(failure)
                undetermined_conversion_devices.append(
                    models.VdoConversionUndeterminedDevice(
                        name=lsblk.name,
                        check_failed=True,
                        failure=failure
                    )
                )
                continue
            result = _check_vdo_pre_conversion(device)
            if result not in (255, 0, 1):
                failure = (
                    'unexpected error from \'vdoprepareforlvm\' for {0}; result = {1}'
                    .format(lsblk.name, result)
                )
                undetermined_conversion_devices.append(
                    models.VdoConversionUndeterminedDevice(
                        name=lsblk.name,
                        check_failed=True,
                        failure=failure
                    )
                )
                continue

            if result == 255:
                # Not a vdo.
                continue

            if result:
                pre_conversion_devices.append(
                  models.VdoConversionPreDevice(name=lsblk.name))
            else:
                result = _check_vdo_lvm_managed(device)
                failure = (None if result in (0, 2) else
                           'unexpected error from \'blkid\' for {0}; '
                           'result = {1}'.format(lsblk.name, result))

                post_conversion_devices.append(
                  models.VdoConversionPostDevice(name=lsblk.name,
                                                 complete=(not result),
                                                 check_failed=(failure is not None),
                                                 failure=failure))

    return models.VdoConversionInfo(pre_conversion=pre_conversion_devices,
                                    post_conversion=post_conversion_devices,
                                    undetermined_conversion=undetermined_conversion_devices)



import pytest
import pyudev
import six
from mock import mock_open, patch

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.actor import biosdevname
from leapp.libraries.stdlib import api
from leapp.models import Interface, PCIAddress


class LoggerMocked(object):
    def __init__(self):
        self.infomsg = None

    def info(self, *args):
        self.infomsg = args

    def __call__(self):
        return self


def test_biosdevname_disabled(monkeypatch):
    mock_config = mock_open(read_data="biosdevname=0")
    with patch("builtins.open" if six.PY3 else "__builtin__.open", mock_config):
        assert biosdevname.is_biosdevname_disabled()


def test_biosdevname_enabled(monkeypatch):
    mock_config = mock_open(read_data="biosdevname=1")
    with patch("builtins.open" if six.PY3 else "__builtin__.open", mock_config):
        assert not biosdevname.is_biosdevname_disabled()


class pyudev_enum_mock(object):
    def __init__(self, vendor):
        self.vendor = vendor

    def match_sys_name(self, _):
        class dev(object):
            attributes = {'sys_vendor': self.vendor}

        return [dev()]

    def match_subsystem(self, _):
        return self

    def __call__(self, _):
        return self


def test_is_vendor_is_dell(monkeypatch):
    monkeypatch.setattr(pyudev, "Enumerator", pyudev_enum_mock("Dell"))
    assert biosdevname.is_vendor_dell()


def test_is_vendor_is_not_dell(monkeypatch):
    monkeypatch.setattr(pyudev, "Enumerator", pyudev_enum_mock("HP"))
    assert not biosdevname.is_vendor_dell()


def test_all_interfaces_biosdevname(monkeypatch):
    pci_info = PCIAddress(domain="domain", function="function", bus="bus", device="device")

    interfaces = [
        Interface(
            name="eth0", mac="mac", vendor="dell", pci_info=pci_info, devpath="path", driver="drv"
        )
    ]
    assert not biosdevname.all_interfaces_biosdevname(interfaces)
    interfaces = [
        Interface(
            name="em0", mac="mac", vendor="dell", pci_info=pci_info, devpath="path", driver="drv"
        )
    ]
    assert biosdevname.all_interfaces_biosdevname(interfaces)
    interfaces = [
        Interface(
            name="p0p22", mac="mac", vendor="dell", pci_info=pci_info, devpath="path", driver="drv"
        )
    ]
    assert biosdevname.all_interfaces_biosdevname(interfaces)

    interfaces = [
        Interface(
            name="p1p2", mac="mac", vendor="dell", pci_info=pci_info, devpath="path", driver="drv"
        ),
        Interface(
            name="em2", mac="mac", vendor="dell", pci_info=pci_info, devpath="path", driver="drv"
        ),
    ]
    assert biosdevname.all_interfaces_biosdevname(interfaces)

    interfaces = [
        Interface(
            name="p1p2", mac="mac", vendor="dell", pci_info=pci_info, devpath="path", driver="drv"
        ),
        Interface(
            name="em2", mac="mac", vendor="dell", pci_info=pci_info, devpath="path", driver="drv"
        ),
        Interface(
            name="eth0", mac="mac", vendor="dell", pci_info=pci_info, devpath="path", driver="drv"
        ),
    ]
    assert not biosdevname.all_interfaces_biosdevname(interfaces)


def test_enable_biosdevname(monkeypatch):
    result = []
    monkeypatch.setattr(api, 'current_logger', LoggerMocked())
    monkeypatch.setattr(api, 'produce', result.append)

    biosdevname.enable_biosdevname()
    assert (
        "Biosdevname naming scheme in use, explicitly enabling biosdevname on the target RHEL system"
        in api.current_logger.infomsg
    )
    assert result[0].key == "biosdevname"
    assert result[0].value == "1"


def test_check_biosdevname(monkeypatch):
    def persistent_net_names_mocked(*models):
        yield None

    monkeypatch.setattr(api, "consume", persistent_net_names_mocked)
    monkeypatch.setattr(biosdevname, "is_biosdevname_disabled", lambda: False)
    with pytest.raises(StopActorExecutionError):
        biosdevname.check_biosdevname()



import os

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common.gpg import get_gpg_fp_from_file, get_path_to_gpg_certs, get_pubkeys_from_rpms
from leapp.libraries.stdlib import api
from leapp.models import GpgKey, InstalledRPM, TrustedGpgKeys


def _get_pubkeys(installed_rpms):
    """
    Get pubkeys from installed rpms and the trusted directory
    """
    pubkeys = get_pubkeys_from_rpms(installed_rpms)
    db_pubkeys = [key.fingerprint for key in pubkeys]
    certs_path = get_path_to_gpg_certs()
    for certname in os.listdir(certs_path):
        key_file = os.path.join(certs_path, certname)
        fps = get_gpg_fp_from_file(key_file)
        for fp in fps:
            if fp not in db_pubkeys:
                pubkeys.append(GpgKey(fingerprint=fp, rpmdb=False, filename=key_file))
                db_pubkeys += fp
    return pubkeys


def process():
    """
    Process keys in RPM DB and the ones in trusted directory to produce a list of trusted keys
    """

    try:
        installed_rpms = next(api.consume(InstalledRPM))
    except StopIteration:
        raise StopActorExecutionError(
            'Could not check for valid GPG keys', details={'details': 'No InstalledRPM facts'}
        )
    pubkeys = _get_pubkeys(installed_rpms)
    api.produce(TrustedGpgKeys(items=pubkeys))



from leapp.actors import Actor
from leapp.libraries.actor import copydnfconfintotargetuserspace
from leapp.models import TargetUserSpacePreupgradeTasks
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class CopyDNFConfIntoTargetUserspace(Actor):
    """
    Copy dnf.conf into target userspace

    Copies /etc/leapp/files/dnf.conf to target userspace. If it isn't available
    /etc/dnf/dnf.conf is copied instead. This allows specifying a different
    config for the target userspace, which might be required if the source
    system configuration file isn't compatible with the target one. One such
    example is incompatible proxy configuration between RHEL7 and RHEL8 DNF
    versions.
    """
    name = "copy_dnf_conf_into_target_userspace"
    consumes = ()
    produces = (TargetUserSpacePreupgradeTasks,)
    tags = (FactsPhaseTag, IPUWorkflowTag)

    def process(self):
        copydnfconfintotargetuserspace.process()



from leapp.models import fields, Model
from leapp.topics import TransactionTopic


class RepoMapEntry(Model):
    topic = TransactionTopic

    source = fields.String()
    """The source PES id."""

    target = fields.List(fields.String())
    """List of target PES ids"""


class PESIDRepositoryEntry(Model):
    """
    Represent metadata about particular repository.

    The metadata are used to identify purpose and nature of the repository.

    Warning: This model is not expected to be consumed/produced by any actor
    directly. As well, it is not covered by deprecation process and can be
    changed or removed any time.
    """

    topic = TransactionTopic

    pesid = fields.String()
    """
    The PES id of the repository.

    The PES id indicate family of YUM repositories. E.g. rhel8-BaseOS covers
    variants of BaseOS YUM repositories for all channels, architectures,
    RHUI, etc, which have basically same purpose.
    """

    major_version = fields.String()
    """
    The major version of OS.

    E.g. for RHEL 7.9 the major version is 7. Since we work with versions
    as with strings throughout the whole codebase, we keep this data type here too.
    """

    repoid = fields.String()
    """
    The repository ID which identifies the repository from YUM/DNF POV.
    """

    arch = fields.StringEnum(['x86_64', 's390x', 'ppc64le', 'aarch64'])
    """
    The architecture for which the repository is delivered.
    """

    repo_type = fields.StringEnum(['rpm', 'debug', 'srpm'])
    """
    The repository type.

    In our case, we usually map just repositories with the 'rpm' type, so
    usually you can see just this one, but the others are possible to add
    too.
    """

    channel = fields.StringEnum(['ga', 'e4s', 'eus', 'aus', 'beta'])
    """
    The 'channel' of the repository.

    The 'channel' could be a little bit inaccurate term, but let's use it in
    this project. The standard repositories has 'ga' channel. 'beta'
    repositories are unsupported for IPU, however they are useful for testing
    purposes. The other channels indicate premium repositories.
    """

    rhui = fields.StringEnum(['', 'aws', 'azure', 'google', 'alibaba'])
    """
    Specifies what cloud provider (RHUI) is the repository specific to.

    Empty string denotes that the repository is not specific to any cloud provider.
    """


class RepositoriesMapping(Model):
    """
    Private model containing information about mapping between repositories.

    Warning: We expect to be only consumers of this model.
    This means the model is not covered by deprecation process and can be
    changed or removed any time.
    """
    topic = TransactionTopic

    mapping = fields.List(fields.Model(RepoMapEntry), default=[])
    repositories = fields.List(fields.Model(PESIDRepositoryEntry), default=[])



from leapp.libraries.stdlib import api, CalledProcessError, run

BIOS_PATH = '/boot/grub2/grubenv'
EFI_PATH = '/boot/efi/EFI/redhat/grubenv'


def grubenv_to_file():
    try:
        run(['unlink', BIOS_PATH])
    except CalledProcessError as err:
        api.current_logger().warning('Could not unlink {}: {}'.format(BIOS_PATH, str(err)))
        return
    try:
        run(['cp', '-a', EFI_PATH, BIOS_PATH])
        api.current_logger().info(
            '{} converted from being a symlink pointing to {} file into a regular file'.format(BIOS_PATH, EFI_PATH)
        )
    except CalledProcessError as err:
        api.current_logger().warning('Could not copy content of {} to {}: {}'.format(EFI_PATH, BIOS_PATH, str(err)))



from leapp.actors import Actor
from leapp.libraries.common import dnfplugin
from leapp.models import DNFWorkaround
from leapp.tags import IPUWorkflowTag, PreparationPhaseTag


class ApplyTransactionWorkarounds(Actor):
    """
    Executes registered workaround scripts on the system before the upgrade transaction
    """

    name = 'applytransactionworkarounds'
    consumes = (DNFWorkaround,)
    produces = ()
    tags = (IPUWorkflowTag, PreparationPhaseTag)

    def process(self):
        dnfplugin.apply_workarounds()



import os
import shutil
import sys

from leapp.libraries.common.utils import makedirs
from leapp.libraries.stdlib import api

LEAPP_HOME = '/root/tmp_leapp_py3'


def _get_python_dirname():
    # NOTE: I thought about the static value: python2.7 for el7, python3.6 for
    # el8; but in the end I've ratcher switched to this generic solution.
    return 'python{}.{}'.format(sys.version_info.major, sys.version_info.minor)


def _get_orig_leapp_path():
    return os.path.join('/usr/lib', _get_python_dirname(), 'site-packages/leapp')


def apply_python3_workaround():
    py3_leapp = os.path.join(LEAPP_HOME, 'leapp3')
    if os.path.exists(LEAPP_HOME):
        try:
            shutil.rmtree(LEAPP_HOME)
        except OSError as e:
            api.current_logger().error('Could not remove {} directory: {}'.format(LEAPP_HOME, str(e)))

    makedirs(LEAPP_HOME)
    leapp_lib_symlink_path = os.path.join(LEAPP_HOME, 'leapp')
    os.symlink(_get_orig_leapp_path(), leapp_lib_symlink_path)
    with open(py3_leapp, 'w') as f:
        f_content = [
            '#!/usr/bin/python3 -B',
            'import sys',
            'sys.path.append(\'{}\')'.format(LEAPP_HOME),
            '',
            'import leapp.cli',
            'sys.exit(leapp.cli.main())',
        ]
        f.write('{}\n\n'.format('\n'.join(f_content)))
    os.chmod(py3_leapp, 0o770)



from leapp.actors import Actor
from leapp.libraries.common.module import get_enabled_modules
from leapp.models import EnabledModules, Module
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class GetEnabledModules(Actor):
    """
    Provides data about which module streams are enabled on the source system.
    """

    name = 'get_enabled_modules'
    consumes = ()
    produces = (EnabledModules,)
    tags = (IPUWorkflowTag, FactsPhaseTag)

    def process(self):
        modules = [Module(name=m.getName(), stream=m.getStream()) for m in get_enabled_modules()]
        self.produce(EnabledModules(modules=modules))



from leapp.libraries.actor import selinuxprepare
from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import SELinuxModule, SELinuxModules


class run_mocked(object):
    def __init__(self):
        self.args = []
        self.called = 0
        self.removed_modules = set()
        self.non_semodule_calls = 0

    def __call__(self, args, split=True):
        self.called += 1
        self.args = args

        if self.args[0] == 'semodule':
            stdout = [
                'libsemanage.semanage_direct_remove_key: Removing last dummy module '
                + '(no other dummy module exists at another priority).'
            ]
            for idx, value in enumerate(self.args):
                if value == "-r":
                    self.removed_modules.add(self.args[idx + 1])
        else:
            self.non_semodule_calls += 1

        return {'stdout': stdout}


def test_remove_custom_modules(monkeypatch):
    mock_modules = {'a': 99, 'b': 300, 'c': 400, 'abrt': 190}
    mock_templates = {'base_container': 400, 'net_container': 300}

    def consume_SELinuxModules_mocked(*models):

        semodule_list = [SELinuxModule(name=name, priority=priority, content='', removed=[])
                         for name, priority in mock_modules.items()]
        template_list = [SELinuxModule(name=name, priority=priority, content='', removed=[])
                         for name, priority in mock_templates.items()]
        yield SELinuxModules(modules=semodule_list, templates=template_list)

    monkeypatch.setattr(api, 'consume', consume_SELinuxModules_mocked)
    monkeypatch.setattr(selinuxprepare, 'run', run_mocked())

    selinuxprepare.remove_custom_modules()
    # 1 call for udica templates and 1 for other custom modules
    assert selinuxprepare.run.called == 2
    assert selinuxprepare.run.non_semodule_calls == 0
    # verify that remove_custom_modules tried to remove all given modules
    assert (set(mock_modules).union(set(mock_templates)) - selinuxprepare.run.removed_modules) == set()



from leapp.actors import Actor
from leapp.libraries.actor import opensshuseprivilegeseparationcheck
from leapp.models import OpenSshConfig, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class OpenSshUsePrivilegeSeparationCheck(Actor):
    """
    UsePrivilegeSeparation configuration option was removed.

    Check the value of UsePrivilegeSeparation in OpenSSH server config file
    and warn about its deprecation if it is set to non-default value.
    """
    name = 'open_ssh_use_privilege_separation'
    consumes = (OpenSshConfig, )
    produces = (Report, )
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        opensshuseprivilegeseparationcheck.process(self.consume(OpenSshConfig))



from leapp.actors import Actor
from leapp.libraries.actor.checkfstabmountorder import check_fstab_mount_order
from leapp.models import StorageInfo
from leapp.reporting import Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckFstabMountOrder(Actor):
    """
    Checks order of entries in /etc/fstab based on their mount point and inhibits upgrade if overshadowing is detected.
    """

    name = "check_fstab_mount_order"
    consumes = (StorageInfo,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag,)

    def process(self):
        check_fstab_mount_order()



from leapp.models import fields, Model
from leapp.topics import SystemFactsTopic


class InstalledDesktopsFacts(Model):
    """
    The model includes fact about installed
    """
    topic = SystemFactsTopic
    gnome_installed = fields.Boolean(default=False)
    kde_installed = fields.Boolean(default=False)



import errno

from leapp.libraries.actor.vsftpdconfigupdate import migrate_configs
from leapp.libraries.common.testutils import make_IOError
from leapp.libraries.common.vsftpdutils import VSFTPD_DEFAULT_CONFIG_PATH
from leapp.models import VsftpdConfig, VsftpdFacts


class MockFileOperations(object):
    def __init__(self):
        self.files = {}
        self.files_read = {}
        self.files_written = {}
        self.read_called = 0
        self.write_called = 0

    def _increment_read_counters(self, path):
        self.read_called += 1
        self.files_read.setdefault(path, 0)
        self.files_read[path] += 1

    def read(self, path):
        self._increment_read_counters(path)
        try:
            return self.files[path]
        except KeyError:
            raise make_IOError(errno.ENOENT)

    def _increment_write_counters(self, path):
        self.write_called += 1
        self.files_written.setdefault(path, 0)
        self.files_written[path] += 1

    def write(self, path, content):
        self._increment_write_counters(path)
        self.files[path] = content


def test_restoring_default_config():
    content = 'anonymous_enable=NO\n' \
              'tcp_wrappers=NO\n' \
              'strict_ssl_read_eof=NO\n'
    fileops = MockFileOperations()
    fileops.files[VSFTPD_DEFAULT_CONFIG_PATH] = content
    config = VsftpdConfig(path=VSFTPD_DEFAULT_CONFIG_PATH,
                          tcp_wrappers=False, strict_ssl_read_eof=False)
    facts = VsftpdFacts(default_config_hash='foobar', configs=[config])

    migrate_configs(facts, fileops=fileops)

    assert len(fileops.files_read) == 1
    assert VSFTPD_DEFAULT_CONFIG_PATH in fileops.files_read
    assert len(fileops.files_written) == 1
    assert VSFTPD_DEFAULT_CONFIG_PATH in fileops.files_written
    expected_lines = ['# Commented out by Leapp:',
                      '#anonymous_enable=NO',
                      'tcp_wrappers=NO',
                      'strict_ssl_read_eof=NO',
                      '',
                      '# Added by Leapp:',
                      'anonymous_enable=YES',
                      '']
    assert fileops.files[VSFTPD_DEFAULT_CONFIG_PATH] == '\n'.join(expected_lines)


def test_setting_tcp_wrappers():
    path = '/etc/vsftpd/foo.conf'
    content = 'tcp_wrappers=YES\n' \
              'strict_ssl_read_eof=NO\n'
    fileops = MockFileOperations()
    fileops.files[path] = content
    config = VsftpdConfig(path=path,
                          tcp_wrappers=True, strict_ssl_read_eof=False)
    facts = VsftpdFacts(configs=[config])

    migrate_configs(facts, fileops=fileops)

    assert path in fileops.files_read
    assert len(fileops.files_written) == 1
    assert path in fileops.files_written
    expected_lines = ['# Commented out by Leapp:',
                      '#tcp_wrappers=YES',
                      'strict_ssl_read_eof=NO',
                      '',
                      '# Added by Leapp:',
                      'tcp_wrappers=NO',
                      '']
    assert fileops.files[path] == '\n'.join(expected_lines)


def test_setting_strict_ssl_read_eof():
    path = '/etc/vsftpd/bar.conf'
    content = 'local_enable=YES\n'
    fileops = MockFileOperations()
    fileops.files[path] = content
    config = VsftpdConfig(path=path,
                          tcp_wrappers=None, strict_ssl_read_eof=None)
    facts = VsftpdFacts(configs=[config])

    migrate_configs(facts, fileops=fileops)

    assert path in fileops.files_read
    assert len(fileops.files_written) == 1
    assert path in fileops.files_written
    expected_lines = ['local_enable=YES',
                      '',
                      '# Added by Leapp:',
                      'strict_ssl_read_eof=NO',
                      '']
    assert fileops.files[path] == '\n'.join(expected_lines)



import json
import os
import re

from leapp.exceptions import CommandError
from leapp.utils import path

HANA_BASE_PATH = '/hana/shared'
HANA_SAPCONTROL_PATH_X86_64 = 'exe/linuxx86_64/hdb/sapcontrol'
HANA_SAPCONTROL_PATH_PPC64LE = 'exe/linuxppc64le/hdb/sapcontrol'

LEAPP_UPGRADE_FLAVOUR_DEFAULT = 'default'
LEAPP_UPGRADE_FLAVOUR_SAP_HANA = 'saphana'
LEAPP_UPGRADE_PATHS = 'upgrade_paths.json'

VERSION_REGEX = re.compile(r"^([1-9]\d*)\.(\d+)$")


def check_version(version):
    """
    Versioning schema: MAJOR.MINOR
    In case version contains an invalid version string, an CommandError will be raised.

    :raises: CommandError
    :return: release tuple
    """
    if not re.match(VERSION_REGEX, version):
        raise CommandError('Unexpected format of target version: {}'.format(version))
    return version.split('.')[0]


def get_major_version(version):
    """
    Return the major version from the given version string.

    Versioning schema: MAJOR.MINOR.PATCH

    :param str version: The version string according to the versioning schema described.
    :rtype: str
    :returns: The major version from the given version string.
    """
    return str(check_version(version)[0])


def detect_sap_hana():
    """
    Detect SAP HANA based on existence of /hana/shared/*/exe/linuxx86_64/hdb/sapcontrol
    """
    if os.path.exists(HANA_BASE_PATH):
        for entry in os.listdir(HANA_BASE_PATH):
            # Does /hana/shared/{entry}/exe/linuxx86_64/hdb/sapcontrol exist?
            sap_on_intel = os.path.exists(os.path.join(HANA_BASE_PATH, entry, HANA_SAPCONTROL_PATH_X86_64))
            sap_on_power = os.path.exists(os.path.join(HANA_BASE_PATH, entry, HANA_SAPCONTROL_PATH_PPC64LE))
            if sap_on_intel or sap_on_power:
                return True
    return False


def get_upgrade_flavour():
    """
    Returns the flavour of the upgrade for this system.
    """
    if detect_sap_hana():
        return LEAPP_UPGRADE_FLAVOUR_SAP_HANA
    return LEAPP_UPGRADE_FLAVOUR_DEFAULT


def get_os_release_version_id(filepath):
    """
    Retrieve data about System OS release from provided file.

    :return: `str` version_id
    """
    with open(filepath) as f:
        data = dict(l.strip().split('=', 1) for l in f.readlines() if '=' in l)
        return data.get('VERSION_ID', '').strip('"')


def get_upgrade_paths_config():
    # NOTE(ivasilev) Importing here not to have circular dependencies
    from leapp.cli.commands.upgrade import util  # noqa: C415; pylint: disable=import-outside-toplevel

    repository = util.load_repositories_from('repo_path', '/etc/leapp/repo.d/', manager=None)
    with open(path.get_common_file_path(repository, LEAPP_UPGRADE_PATHS)) as f:
        upgrade_paths_map = json.loads(f.read())
    return upgrade_paths_map


def get_target_versions_from_config(src_version_id, flavor):
    """
    Retrieve all possible target versions from upgrade_paths_map.
    If no match is found returns empty list.
    """
    upgrade_paths_map = get_upgrade_paths_config()
    return upgrade_paths_map.get(flavor, {}).get(src_version_id, [])


def get_supported_target_versions(flavour=get_upgrade_flavour()):
    """
    Return a list of supported target versions for the given `flavour` of upgrade.
    The default value for `flavour` is `default`.
    """

    current_version_id = get_os_release_version_id('/etc/os-release')
    target_versions = get_target_versions_from_config(current_version_id, flavour)
    if not target_versions:
        # If we cannot find a particular major.minor version in the map,
        # we fallback to pick a target version just based on a major version.
        # This can happen for example when testing not yet released versions
        major_version = get_major_version(current_version_id)
        target_versions = get_target_versions_from_config(major_version, flavour)

    return target_versions


def get_target_version(flavour):
    target_versions = get_supported_target_versions(flavour)
    return target_versions[-1] if target_versions else None


def vet_upgrade_path(args):
    """
    Make sure the user requested upgrade_path is a supported one.
    If LEAPP_DEVEL_TARGET_RELEASE is set then it's value is not vetted against upgrade_paths_map but used as is.

    :raises: `CommandError` if the specified upgrade_path is not supported
    :return: `tuple` (target_release, flavor)
    """
    flavor = get_upgrade_flavour()
    env_version_override = os.getenv('LEAPP_DEVEL_TARGET_RELEASE')
    if env_version_override:
        check_version(env_version_override)
        return (env_version_override, flavor)
    target_release = args.target or get_target_version(flavor)
    supported_target_versions = get_supported_target_versions(flavor)
    if target_release not in supported_target_versions:
        raise CommandError(
                "Upgrade to {to} for {flavor} upgrade path is not supported, possible choices are {choices}".format(
                    to=target_release,
                    flavor=flavor,
                    choices=','.join(supported_target_versions)))
    return (target_release, flavor)



from leapp.actors import Actor
from leapp.libraries.actor import opensshalgorithmscheck
from leapp.models import OpenSshConfig, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class OpenSshAlgorithmsCheck(Actor):
    """
    OpenSSH configuration does not contain any unsupported cryptographic algorithms.

    Check the values of Ciphers and MACs in OpenSSH server config file and warn
    about removed algorithms which might cause the server to fail to start.
    """
    name = 'open_ssh_algorithms'
    consumes = (OpenSshConfig,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        opensshalgorithmscheck.process(self.consume(OpenSshConfig))



import errno
import os
import re

import leapp.libraries.common.spamassassinutils as lib
from leapp.libraries.stdlib import api


def spamassassin_service_overriden(listdir):
    file_name = os.path.basename(lib.SPAMASSASSIN_SERVICE_OVERRIDE)
    dir_path = os.path.dirname(lib.SPAMASSASSIN_SERVICE_OVERRIDE)
    try:
        return file_name in listdir(dir_path)
    except OSError as e:
        return e.errno != errno.ENOENT


def _parse_ssl_version(content):
    _, assignment, _ = lib.parse_sysconfig_spamassassin(content)
    if re.search(r'(?<![\w-])--ssl-version(=|\s+)sslv3(?![\w-])', assignment):
        return 'sslv3'
    if re.search(r'(?<![\w-])--ssl-version(=|\s+)tlsv1(?![\w-])', assignment):
        return 'tlsv1'
    return None


def get_spamd_ssl_version(read_func):
    """
    Extracts and returns the argument given to the --ssl-version option from
    the spamassassin sysconfig file. Returns None if the option is not specified
    or the file does not exist.
    """
    try:
        content = read_func(lib.SYSCONFIG_SPAMASSASSIN)
    except (IOError, OSError) as e:
        if e.errno != errno.ENOENT:
            api.current_logger().warning(
                'Failed to read spamassassin sysconfig file: %s' % e)
        return None
    return _parse_ssl_version(content)



import os
import re


class PAM(object):
    files = [
        '/etc/pam.d/system-auth',
        '/etc/pam.d/smartcard-auth',
        '/etc/pam.d/password-auth',
        '/etc/pam.d/fingerprint-auth',
        '/etc/pam.d/postlogin'
    ]
    """
    List of system PAM configuration files.
    """

    def __init__(self, config):
        self.modules = self.parse(config)

    def parse(self, config):
        """
        Parse configuration and return list of modules that are present in the
        configuration.
        """
        result = re.findall(
            r"^[ \t]*[^#\s]+.*(pam_\S+)\.so.*$",
            config,
            re.MULTILINE
        )

        return result

    def has(self, module):
        """
        Return True if the module exist in the configuration, False otherwise.
        """
        return module in self.modules

    def has_unknown_module(self, known_modules):
        """
        Return True if the configuration has any module which is not known to
        the caller, False otherwise.
        """
        for module in self.modules:
            if module not in known_modules:
                return True

        return False

    @staticmethod
    def read_file(config):
        """
        Read file contents. Return empty string if the file does not exist.
        """
        if not os.path.isfile(config):
            return ""
        with open(config) as f:
            return f.read()

    @staticmethod
    def from_system_configuration():
        config = ""
        for f in PAM.files:
            config += PAM.read_file(f)

        return PAM(config)



from leapp.libraries.common.tcpwrappersutils import config_applies_to_daemon


def config_affects_daemons(tcp_wrappers_facts, packages_list, daemons):
    """
    Check whether some of the daemons is installed and affected by existing
    configuration of tcp_wrappers based on the.

    :param tcp_wrappers_facts: Facts provided by the TcpWrappersFacts
    :param packages_list: List of packages provided by DistributionSignedRPM
    :param daemons: List of packages and keywords affecting daemons in this format:
                    [{"package-name", ["daemon1", "daemon2", ...], ...}]
    """
    found_packages = set()

    for (package, keywords) in daemons:
        # We do not care for particular daemon if the providing package is not installed
        if package not in packages_list:
            continue

        # Every package can have several daemons or daemons reacting to several keywords
        for daemon in keywords:
            # Is this daemon/keyword affected by the current configuration?
            if not config_applies_to_daemon(tcp_wrappers_facts, daemon):
                continue

            # We do not report particular daemons, but just the high-level list of packages
            found_packages.add(package)

    return found_packages



from leapp import reporting
from leapp.libraries.common.rpms import has_package
from leapp.libraries.stdlib import api
from leapp.models import DistributionSignedRPM

# Summary for bacula-director report
report_director_inst_summary = (
    'Bacula director component will be upgraded. Since the new version is'
    ' incompatible with the current version, it is necessary to proceed'
    ' with additional steps for the complete upgrade of the Bacula backup'
    ' database.'
)

report_director_inst_hint = (
    'Back up your data before proceeding with the upgrade'
    ' and use the command "/usr/libexec/bacula/update_bacula_tables <dbtype>" to upgrade'
    ' the Bacula database after the system upgrade.'
    ' The value of <dbtype> depends on the database backend, possible values are'
    ' sqlite3, mysql, postgresql.'
)


def _report_director_installed():
    """
    Create report on bacula-director package installation detection.

    Should remind user about present Bacula director package
    installation and warn them about necessary additional steps.
    """
    reporting.create_report([
        reporting.Title('bacula (bacula-director) has been detected on your system'),
        reporting.Summary(report_director_inst_summary),
        reporting.Severity(reporting.Severity.MEDIUM),
        reporting.Groups([reporting.Groups.SERVICES]),
        reporting.RelatedResource('package', 'bacula-director'),
        reporting.Remediation(hint=report_director_inst_hint),
        ])


def report_installed_packages(_context=api):
    """
    Create reports according to detected bacula packages.

    Create the report if the bacula-director rpm (RH signed) is installed.
    """
    has_director = has_package(DistributionSignedRPM, 'bacula-director', context=_context)

    if has_director:
        # bacula-director
        _report_director_installed()



from leapp.models import fields, Model
from leapp.topics import SystemFactsTopic


class DefaultGrub(Model):
    """
    A model with '/etc/default/grub' option as key / value
    """
    topic = SystemFactsTopic

    name = fields.String()
    """ Option name, e.g. GRUB_TIMEOUT """
    value = fields.String()
    """ Option value, e.g. '5' in case of GRUB_TIMEOUT=5 """


class DefaultGrubInfo(Model):
    """
    A message with '/etc/default/grub' content
    """

    topic = SystemFactsTopic

    default_grub_info = fields.List(fields.Model(DefaultGrub))
    """ List of '/etc/default/grub' options as key / value """



from leapp import reporting
from leapp.libraries.actor import checkmemcached
from leapp.libraries.common.testutils import create_report_mocked


def test_uninstalled(monkeypatch):
    for sysconfig_default in (False, True):
        for udp_disabled in (False, True):
            monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
            monkeypatch.setattr(checkmemcached, 'is_sysconfig_default', lambda: sysconfig_default, )
            monkeypatch.setattr(checkmemcached, 'is_udp_disabled', lambda: udp_disabled)

            checkmemcached.check_memcached(False)

            assert reporting.create_report.called == 0


def test_installed_defconf(monkeypatch):
    for udp_disabled in (False, True):
        monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
        monkeypatch.setattr(checkmemcached, 'is_sysconfig_default', lambda: True)
        monkeypatch.setattr(checkmemcached, 'is_udp_disabled', lambda: udp_disabled)

        checkmemcached.check_memcached(True)

        assert reporting.create_report.called == 1
        assert reporting.create_report.report_fields['title'] == 'memcached service is using default configuration'


def test_installed_nodefconf_udp(monkeypatch):
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr(checkmemcached, 'is_sysconfig_default', lambda: False)
    monkeypatch.setattr(checkmemcached, 'is_udp_disabled', lambda: False)

    checkmemcached.check_memcached(True)

    assert reporting.create_report.called == 1
    assert reporting.create_report.report_fields['title'] == 'memcached has enabled UDP port'


def test_installed_nodefconf_noudp(monkeypatch):
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr(checkmemcached, 'is_sysconfig_default', lambda: False)
    monkeypatch.setattr(checkmemcached, 'is_udp_disabled', lambda: True)

    checkmemcached.check_memcached(True)

    assert reporting.create_report.called == 1
    assert reporting.create_report.report_fields['title'] == 'memcached has already disabled UDP port'



import pytest

from leapp import reporting
from leapp.libraries.actor.cupscheck import check_include_directive
from leapp.libraries.common.testutils import create_report_mocked
from leapp.models import CupsChangedFeatures


@pytest.mark.parametrize("include_exists,n_reports", [(False, 0), (True, 1)])
def test_check_include_directive(include_exists, n_reports):
    facts = CupsChangedFeatures(include=include_exists,
                                include_files=['/etc/cups/cupsd.conf'])
    report_func = create_report_mocked()

    check_include_directive(facts, report_func)

    assert report_func.called == n_reports

    if report_func.called:
        report_fields = report_func.report_fields

        assert 'no longer supports usage of Include' in report_fields['title']
        assert report_fields['severity'] == reporting.Severity.MEDIUM



from leapp.libraries.actor import checkrhsmsku
from leapp.libraries.common import rhsm
from leapp.libraries.common.testutils import create_report_mocked
from leapp.libraries.stdlib import api
from leapp.models import RHSMInfo
from leapp.utils.report import is_inhibitor


def test_sku_report_skipped(monkeypatch):
    monkeypatch.setattr(rhsm, 'skip_rhsm', lambda: True)
    monkeypatch.setattr(api, 'consume', lambda x: (RHSMInfo(attached_skus=[]),))
    monkeypatch.setattr(checkrhsmsku, 'create_report', create_report_mocked())
    checkrhsmsku.process()
    assert not checkrhsmsku.create_report.called


def test_sku_report_has_skus(monkeypatch):
    monkeypatch.setattr(rhsm, 'skip_rhsm', lambda: False)
    monkeypatch.setattr(api, 'consume', lambda x: (RHSMInfo(attached_skus=['testing-sku']),))
    monkeypatch.setattr(checkrhsmsku, 'create_report', create_report_mocked())
    checkrhsmsku.process()
    assert not checkrhsmsku.create_report.called


def test_sku_report_has_no_skus(monkeypatch):
    monkeypatch.setattr(rhsm, 'skip_rhsm', lambda: False)
    monkeypatch.setattr(api, 'consume', lambda x: (RHSMInfo(attached_skus=[]),))
    monkeypatch.setattr(checkrhsmsku, 'create_report', create_report_mocked())
    checkrhsmsku.process()
    assert checkrhsmsku.create_report.called == 1
    assert checkrhsmsku.create_report.report_fields['title'] == 'The system is not registered or subscribed.'
    assert checkrhsmsku.create_report.report_fields['severity'] == 'high'
    assert is_inhibitor(checkrhsmsku.create_report.report_fields)


def test_sku_report_has_sca(monkeypatch):
    monkeypatch.setattr(rhsm, 'skip_rhsm', lambda: False)
    monkeypatch.setattr(api, 'consume', lambda x: (RHSMInfo(attached_skus=[], sca_detected=True),))
    monkeypatch.setattr(checkrhsmsku, 'create_report', create_report_mocked())
    checkrhsmsku.process()
    assert not checkrhsmsku.create_report.called



import errno
import functools
import grp
import logging
import os
import pwd
import re

import six

from leapp import reporting
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common import repofileutils
from leapp.libraries.common.config import architecture
from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import (
    ActiveKernelModule,
    ActiveKernelModulesFacts,
    DefaultGrub,
    DefaultGrubInfo,
    FirewallsFacts,
    FirewallStatus,
    FirmwareFacts,
    Group,
    GroupsFacts,
    GrubCfgBios,
    KernelModuleParameter,
    RepositoriesFacts,
    SELinuxFacts,
    SysctlVariable,
    SysctlVariablesFacts,
    User,
    UsersFacts
)


def aslist(f):
    """ Decorator used to convert generator to list """
    @functools.wraps(f)
    def inner(*args, **kwargs):
        return list(f(*args, **kwargs))
    return inner


def anyendswith(value, ends):
    """ Check if `value` ends with one of the possible `ends` """
    for end in ends:
        if value.endswith(end):
            return True
    return False


def anyhasprefix(value, prefixes):
    """ Check if `value` starts with on of the possible `prefixes` """
    for p in prefixes:
        if value.startswith(p):
            return True
    return False


@aslist
def _get_system_users():
    skipped_user_names = []
    for p in pwd.getpwall():
        # The /etc/passwd can contain special entries from another service source such as NIS or LDAP. These entries
        # start with + or - sign and might not contain all the mandatory fields, thus are skipped along with other
        # invalid entries for now. The UID and GID fields are always defined by pwd to 0 even when not specifiead in
        # /etc/passwd.
        if p.pw_name != '' and not p.pw_name.startswith(('+', '-')) and p.pw_dir:
            yield User(
                name=p.pw_name,
                uid=p.pw_uid,
                gid=p.pw_gid,
                home=p.pw_dir
            )
        else:
            skipped_user_names.append(p.pw_name)

    if skipped_user_names:
        api.current_logger().debug("These users from /etc/passwd that are special entries for service "
                                   "like NIS, or don't contain all mandatory fields won't be included "
                                   "in UsersFacts: {}".format(skipped_user_names))


def get_system_users_status():
    """ Get a list of users from `/etc/passwd` """
    return UsersFacts(users=_get_system_users())


@aslist
def _get_system_groups():
    skipped_group_names = []
    for g in grp.getgrall():
        # The /etc/group can contain special entries from another service source such as NIS or LDAP. These entries
        # start with + or - sign and might not contain all the mandatory fields, thus are skipped along with other
        # invalid entries for now. The GID field is always defined by pwd to 0 even when not specifiead in
        # /etc/group.
        if g.gr_name != '' and not g.gr_name.startswith(('+', '-')):
            yield Group(
                name=g.gr_name,
                gid=g.gr_gid,
                members=g.gr_mem
            )
        else:
            skipped_group_names.append(g.gr_name)

    if skipped_group_names:
        api.current_logger().debug("These groups from /etc/group that are special entries for service "
                                   "like NIS, or don't contain all mandatory fields won't be included "
                                   "in GroupsFacts: {}".format(skipped_group_names))


def get_system_groups_status():
    """ Get a list of groups from `/etc/groups` """
    return GroupsFacts(groups=_get_system_groups())


@aslist
def _get_active_kernel_modules(logger):
    lines = run(['lsmod'], split=True)['stdout']
    for l in lines[1:]:
        name = l.split(' ')[0]

        # Read parameters of the given module as exposed by the
        # `/sys` VFS, if there are no parameters exposed we just
        # take the name of the module
        base_path = '/sys/module/{module}'.format(module=name)
        parameters_path = os.path.join(base_path, 'parameters')
        if not os.path.exists(parameters_path):
            yield ActiveKernelModule(filename=name, parameters=[])
            continue

        # Use `modinfo` to probe for signature information
        parameter_dict = {}
        try:
            signature = run(['modinfo', '-F', 'signature', name], split=False)['stdout']
        except CalledProcessError:
            signature = None

        signature_string = None
        if signature:
            # Remove whitespace from the signature string
            signature_string = re.sub(r"\s+", "", signature, flags=re.UNICODE)

        # Since we're using the `/sys` VFS we need to use `os.listdir()` to get
        # all the property names and then just read from all the listed paths
        parameters = sorted(os.listdir(parameters_path))
        for param in parameters:
            try:
                with open(os.path.join(parameters_path, param), mode='r') as fp:
                    parameter_dict[param] = fp.read().strip()
            except IOError as exc:
                # Some parameters are write-only, in that case we just log the name of parameter
                # and the module and continue
                if exc.errno in (errno.EACCES, errno.EPERM):
                    msg = 'Unable to read parameter "{param}" of kernel module "{name}"'
                    logger.warning(msg.format(param=param, name=name))
                else:
                    raise exc

        # Project the dictionary as a list of key values
        items = [
            KernelModuleParameter(name=k, value=v)
            for (k, v) in six.iteritems(parameter_dict)
        ]

        yield ActiveKernelModule(
            filename=name,
            parameters=items,
            signature=signature_string
        )


def get_active_kernel_modules_status(logger):
    """ Get a list of active kernel modules """
    return ActiveKernelModulesFacts(kernel_modules=_get_active_kernel_modules(logger))


@aslist
def _get_sysctls():
    unstable = ('fs.dentry-state', 'fs.file-nr', 'fs.inode-nr',
                'fs.inode-state', 'kernel.random.uuid', 'kernel.random.entropy_avail',
                'kernel.ns_last_pid', 'net.netfilter.nf_conntrack_count',
                'net.netfilter.nf_conntrack_events', 'kernel.sched_domain.',
                'dev.cdrom.info', 'kernel.pty.nr')

    variables = []
    for sc in run(['sysctl', '-a'], split=True)['stdout']:
        name = sc.split(' ', 1)[0]
        # if the sysctl name has an unstable prefix, we skip
        if anyhasprefix(name, unstable):
            continue
        variables.append(sc)

    # sort our variables so they can be diffed directly when needed
    for var in sorted(variables):
        name, value = tuple(map(type(var).strip, var.split('=', 1)))
        yield SysctlVariable(
            name=name,
            value=value
        )


def get_sysctls_status():
    r""" Get a list of stable `sysctls` variables

        Note that some variables are inherently unstable and we need to blacklist
        them:

        diff -u <(sysctl -a 2>/dev/null | sort) <(sysctl -a 2>/dev/null | sort)\
                | grep -E '^\+[a-z]'\
                | cut -d' ' -f1\
                | cut -d+ -f2
    """
    return SysctlVariablesFacts(sysctl_variables=_get_sysctls())


def get_repositories_status():
    """ Get a basic information about YUM repositories installed in the system """
    return RepositoriesFacts(repositories=repofileutils.get_parsed_repofiles())


def get_selinux_status():
    """ Get SELinux status information """
    # will be None if something went wrong or contain SELinuxFacts otherwise
    res = None
    try:
        import selinux  # pylint: disable=import-outside-toplevel
    except ImportError:
        api.report_error("SELinux Import Error", details="libselinux-python package must be installed.")
        return res

    outdata = dict({'enabled': selinux.is_selinux_enabled() == 1})
    outdata['mls_enabled'] = selinux.is_selinux_mls_enabled() == 1

    try:
        outdata['runtime_mode'] = "enforcing" if selinux.security_getenforce() == 1 else "permissive"
        # FIXME: check selinux_getenforcemode[0] (that should be return value of a underneath function)
        enforce_mode = selinux.selinux_getenforcemode()[1]
        if enforce_mode >= 0:
            outdata['static_mode'] = "enforcing" if enforce_mode == 1 else "permissive"
        else:
            outdata['static_mode'] = "disabled"
        outdata['policy'] = selinux.selinux_getpolicytype()[1]
    except OSError:
        # This happens when SELinux is disabled
        # [Errno 2] No such file or directory
        outdata['runtime_mode'] = 'permissive'
        outdata['static_mode'] = 'disabled'
        outdata['policy'] = 'targeted'

    res = SELinuxFacts(**outdata)
    return res


def get_firewalls_status():
    """ Get firewalld status information """
    logger = logging.getLogger('get_firewalld_status')

    def _get_firewall_status(service_name):
        try:
            ret_list = run(['systemctl', 'is-active', service_name], split=True)['stdout']
            active = ret_list[0] == 'active'
        except CalledProcessError:
            active = False
            logger.debug('The %s service is likely not active', service_name)

        try:
            ret_list = run(['systemctl', 'is-enabled', service_name], split=True)['stdout']
            enabled = ret_list[0] == 'enabled'
        except CalledProcessError:
            enabled = False
            logger.debug('The %s service is likely not enabled nor running', service_name)

        return FirewallStatus(
            active=active,
            enabled=enabled,
        )

    return FirewallsFacts(
        firewalld=_get_firewall_status('firewalld'),
        iptables=_get_firewall_status('iptables'),
        ip6tables=_get_firewall_status('ip6tables'),
    )


def get_firmware():
    firmware = 'efi' if os.path.isdir('/sys/firmware/efi') else 'bios'
    if architecture.matches_architecture(architecture.ARCH_PPC64LE):
        ppc64le_opal = bool(os.path.isdir('/sys/firmware/opal/'))
        return FirmwareFacts(firmware=firmware, ppc64le_opal=ppc64le_opal)
    return FirmwareFacts(firmware=firmware)


@aslist
def _default_grub_info():
    default_grb_fpath = '/etc/default/grub'
    if not os.path.isfile(default_grb_fpath):
        reporting.create_report([
            reporting.Title('File "{}" does not exist!'.format(default_grb_fpath)),
            reporting.Summary(
                'Leapp detected "{}" does not exist. The file is essential for the in-place upgrade '
                'to finish successfully. This scenario might have occurred if the system was already '
                'upgraded from RHEL 6. Please re-create the file manually.'.format(default_grb_fpath)
            ),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Groups([reporting.Groups.INHIBITOR]),
            reporting.Groups([reporting.Groups.BOOT]),
            reporting.RelatedResource('file', default_grb_fpath),
            reporting.ExternalLink(
                url='https://access.redhat.com/solutions/3185891',
                title='How to re-create the missing "{}" file in Red Hat Enterprise Linux 7?'.format(
                    default_grb_fpath
                )
            ),
        ])
    else:
        for line in run(['cat', default_grb_fpath], split=True)['stdout']:
            line = line.strip()
            if not line or line[0] == '#':
                # skip comments and empty lines
                continue
            try:
                name, value = tuple(map(type(line).strip, line.split('=', 1)))
            except ValueError as e:
                # we do not want to really continue when we cannot parse this file
                # TODO(pstodulk): rewrite this in the form we produce inhibitor
                # with problematic lines. This is improvement just in comparison
                # to the original hard crash.
                raise StopActorExecutionError(
                    'Failed parsing of {}'.format(default_grb_fpath),
                    details={
                        'error': str(e),
                        'problematic line': str(line)
                    }
                )

            yield DefaultGrub(
                name=name,
                value=value
            )


def get_default_grub_conf():
    """ Get a list of GRUB parameters from /etc/default/grub """
    return DefaultGrubInfo(default_grub_info=_default_grub_info())


def get_bios_grubcfg_details():
    """ Get BIOS (non-EFI) Grub config details """
    if get_firmware().firmware == 'bios' and not architecture.matches_architecture(architecture.ARCH_S390X):
        with open('/boot/grub2/grub.cfg') as fo:
            content = fo.read()
        insmod_bls = bool('insmod blscfg' in content)
        return GrubCfgBios(insmod_bls=insmod_bls)
    return None
